#! /usr/bin/env python
import sys
from harpoon.commands.base import Command
from github import Github, UnknownObjectException


class CommandGithub(Command):
    """
    # Github

    **Request Github API**

    * Get information on a repository: `harpoon github repo kneufeld/consolemd`
    * Search for information in Github code: `harpoon github search randhome.io`
    * Search for information in Github repo: `harpoon github search -t repo harpoon`
    """
    name = "github"
    description = "Request Github information through the API"
    config = { 'Github': ['token']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='SubCommands')
        parser_a = subparsers.add_parser('search', help='Search in github')
        parser_a.add_argument('--type', '-t',
                choices=['repo', 'code', 'commit', 'issues', 'wikis', 'users'],
                default='code',
                help='Type of data to search')
        parser_a.add_argument('--limit', '-l',
                default='10', type=int,
                help='Result limit')
        parser_a.add_argument('SEARCH')
        parser_a.set_defaults(subcommand='search')
        parser_b = subparsers.add_parser('repo', help='Information on a github repository')
        parser_b.add_argument('REPOSITORY')
        parser_b.add_argument('-o', '--only-emails', action="store_true", help="Only list emails of committers")
        parser_b.set_defaults(subcommand='repo')

        self.parser = parser

    def run(self, conf, args, plugins):
        g = Github(conf['Github']['token'])
        if 'subcommand' in args:
            if args.subcommand == 'search':
                if args.type == 'code':
                    res = g.search_code(args.SEARCH)
                    nb = 0
                    for i in res:
                        print('[+] %s' % i.html_url)
                        print(i.decoded_content[:300])
                        print('')
                        nb += 1
                        if nb > args.limit:
                            sys.exit(0)
                elif args.type == 'repo':
                    res = g.search_repositories(args.SEARCH)
                    nb = 0
                    for i in res:
                        print('[+] %s by %s' % (i.name, i.owner.name))
                        print('\t%s' % i.description)
                        print('\t%s' % i.html_url)
                        print('\t%s' % i.language)
                        print('\t[Watch: %i][Stars: %i][Forks: %i]' % (
                                i.watchers,
                                i.stargazers_count,
                                i.forks_count
                            )
                        )
                        print('')
                        nb += 1
                        if nb > args.limit:
                            sys.exit(0)
                elif args.type == 'commit':
                    # Not yet implemented by PyGithub
                    raise Exception('Not yet implemented')
            elif args.subcommand == "repo":
                # clean input
                if args.REPOSITORY.startswith("https://github.com"):
                    rep_name = args.REPOSITORY[19:]
                else:
                    rep_name = args.REPOSITORY
                if rep_name.endswith(".git"):
                    rep_name = rep_name[:-4]
                repo = g.get_repo(rep_name)
                # Check if found
                try:
                    idd = repo.id
                except UnknownObjectException:
                    print("Repository not found")
                    return
                if args.only_emails:
                    # FIXME: really slow
                    committers = set()
                    for c in repo.get_commits():
                        if c.committer:
                            if c.committer.email:
                                committers.add(c.committer.email)
                    for c in committers:
                        print(c)
                else:
                    print("-Name: %s" % repo.full_name)
                    print("-Owner: %s %s" % (repo.owner.login, repo.owner.email))
                    print("-Language: %s" % repo.language)
                    print("-%i Watchers / %i Stars / %i Forks" % (
                            repo.watchers,
                            repo.stargazers_count,
                            repo.forks_count
                        )
                    )
                    # FIXME: really slow
                    committers = {}
                    for c in repo.get_commits():
                        if c.committer:
                            if c.committer.email:
                                if c.committer.email in committers:
                                    committers[c.committer.email] += 1
                                else:
                                    committers[c.committer.email] = 1
                    print("-Committers:")
                    for committer in sorted(committers.items()):
                        print("\t%s %i" % ( committer[0], committer[1]))


            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import operator
import json
from harpoon.commands.base import Command
from harpoon.lib.utils import json_serial, unbracket
from pycrtsh import Crtsh
from collections import Counter
from datetime import date, datetime


class CommandCert(Command):
    """
    # Crt.sh

    **Search and download information from https://crt.sh/

    * Search all certificates from a domain: `harpoon crtsh domain www.citizenlab.org`
    * Search all certificates from a list of domains in a file with CSV output: `harpoon crtsh list FILE -f CSV`
    * Search for subdomains of a domain : `harpoon crtsh subdomains DOMAIN`
    """
    name = "crtsh"
    description = "Search in https://crt.sh/ (Certificate Transparency database)"

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('domain', help='Get certificates of a domain')
        parser_a.add_argument('DOMAIN', help='Domain')
        parser_a.add_argument('--format', '-f', choices=["csv", "json", "txt"], default="txt", help='Output format (default is txt)')
        parser_a.set_defaults(subcommand='domain')
        parser_b = subparsers.add_parser('list', help='Get certificates for a list of domains')
        parser_b.add_argument('FILE', help='File containing a list of domains')
        parser_b.add_argument('--format', '-f', choices=["csv", "json", "txt"], default="txt", help='Output format (default is txt)')
        parser_b.set_defaults(subcommand='list')
        parser_c = subparsers.add_parser('subdomains', help='Search subdomains of a domain through certificates')
        parser_c.add_argument('DOMAIN', help='Domain to be searched')
        parser_c.set_defaults(subcommand='subdomains')
        self.parser = parser

    def get_subdomains(self, domain):
        crt = Crtsh()
        subdomains = set()
        index = crt.search(unbracket(domain))
        for c in index:
            data = crt.get(c["id"], type="id")
            subdomains.add(data["subject"]["commonName"])
            if "alternative_names" in data["extensions"]:
                for d in data["extensions"]["alternative_names"]:
                    subdomains.add(d)
        return list(subdomains)

    def run(self, conf, args, plugins):
        crt = Crtsh()
        if 'subcommand' in args:
            if args.subcommand == 'domain':
                index = crt.search(unbracket(args.DOMAIN))
                if args.format == "txt":
                    domains = []
                    print("Certificates")
                    for c in index:
                        data = crt.get(c["id"], type="id")
                        print("%s\t%s\t%s\t%s" % (
                            data["subject"]["commonName"],
                            data["not_before"].isoformat(),
                            data["not_after"].isoformat(),
                            data["sha1"]
                            )
                        )
                        if "alternative_names" in data["extensions"]:
                            domains += list(set([a[2:] if a.startswith("*.") else a for a in data["extensions"]["alternative_names"]]))

                    print("\nDomains")
                    count = Counter(domains)
                    for d in sorted(count.items(), key=operator.itemgetter(1), reverse=True):
                        print("-%s: %i occurences" % (d[0], d[1]))
                elif args.format == "json":
                    certs = {}
                    for c in index:
                        certs[c["id"]] = crt.get(c["id"], type="id")
                    print(json.dumps(certs, sort_keys=True, indent=4, separators=(',', ': '), default=json_serial))
                elif args.format == "csv":
                    print("id|serial|sha1|Common Name|Issuer|Not Before|Not After|Basic Constraints|Alt Names")
                    for c in index:
                        data = crt.get(c["id"], type="id")
                        print("%s|%s|%s|%s|%s|%s|%s|%s|%s" % (
                            c["id"],
                            data["serial"],
                            data["sha1"],
                            data["subject"]["commonName"],
                            data["issuer"]["commonName"],
                            data["not_before"],
                            data["not_after"],
                            data["extensions"]["basic_constraints"] if "basic_constraints" in data["extensions"] else "False",
                            ", ".join(data["extensions"]["alternative_names"]) if "alternative_names" in data["extensions"] else ""
                            )
                        )
            elif args.subcommand == 'list':
                sha1_list = []
                with open(args.FILE, 'r') as f:
                    domains = [a.strip() for a in f.read().split()]
                if args.format == "txt":
                    for d in domains:
                        index = crt.search(d)
                        print("Certificates for %s" % d)
                        for c in index:
                            data = crt.get(c["id"], type="id")
                            print("%s\t%s\t%s\t%s" % (
                                    data["subject"]["commonName"],
                                    data["not_before"].isoformat(),
                                    data["not_after"].isoformat(),
                                    data["sha1"]
                                )
                            )
                        print("")
                elif args.format == "json":
                    data = {}
                    for d in domains:
                        data[d] = {}
                        index = crt.search(d)
                        for c in index:
                            if data["sha1"] not in sha1_list:
                                sha1_list.append(data["sha1"])
                                data[d][c["id"]] = crt.get(c["id"], type="id")
                    print(json.dumps(data, sort_keys=True, indent=4, separators=(',', ': '), default=json_serial))
                elif args.format == "csv":
                    print("id|serial|sha1|Common Name|Issuer|Not Before|Not After|Basic Constraints|Alt Names")
                    for d in domains:
                        index = crt.search(d)
                        for c in index:
                            data = crt.get(c["id"], type="id")
                            if data["sha1"] not in sha1_list:
                                sha1_list.append(data["sha1"])
                                print("%s|%s|%s|%s|%s|%s|%s|%s|%s" % (
                                    c["id"],
                                    data["serial"],
                                    data["sha1"],
                                    data["subject"]["commonName"],
                                    data["issuer"]["commonName"],
                                    data["not_before"],
                                    data["not_after"],
                                    data["extensions"]["basic_constraints"] if "basic_constraints" in data["extensions"] else "False",
                                    ", ".join(data["extensions"]["alternative_names"]) if "alternative_names" in data["extensions"] else ""
                                    )
                                )
            elif args.subcommand == 'subdomains':
                for d in self.get_subdomains(unbracket(args.DOMAIN)):
                    print(d)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import os
import json
import datetime
import urllib.request
import tarfile
import geoip2.database
import re
import subprocess
import glob
import shutil
import pyasn
import urllib
import socket
import requests
import pytz
from IPy import IP
from dateutil.parser import parse
from harpoon.commands.base import Command
from harpoon.lib.utils import bracket, unbracket, is_ip
from harpoon.lib.robtex import Robtex, RobtexError
from OTXv2 import OTXv2, IndicatorTypes
from virus_total_apis import PublicApi, PrivateApi
from pygreynoisev1 import GreyNoise, GreyNoiseError
from passivetotal.libs.dns import DnsRequest
from passivetotal.libs.enrichment import EnrichmentRequest
from pythreatgrid2 import ThreatGrid, ThreatGridError
from harpoon.commands.asn import CommandAsn
from mispy import MispServer
from pybinaryedge import BinaryEdge, BinaryEdgeException, BinaryEdgeNotFound
from threatminer import ThreatMiner


class CommandIp(Command):
    """
    # IP

    **Gathers information on an IP address**

    Get information on an IP:
    ```
harpoon ip info 172.34.127.2
MaxMind: Located in None, United States
MaxMind: ASN21928, T-Mobile USA, Inc.
ASN 21928 - T-MOBILE-AS21928 - T-Mobile USA, Inc., US (range 172.32.0.0/11)

Censys:     https://censys.io/ipv4/172.34.127.2
Shodan:     https://www.shodan.io/host/172.34.127.2
IP Info:    http://ipinfo.io/172.34.127.2
BGP HE:     https://bgp.he.net/ip/172.34.127.2
IP Location:    https://www.iplocation.net/?query=172.34.127.2
    ```

    * Get intelligence information on an IP: `harpoon ip intel IP`

    """
    name = "ip"
    description = "Gather information on an IP address"
    config = None
    update_needed = True
    geocity = "/usr/share/GeoIP/GeoLite2-City.mmdb"
    geoasn = "/usr/share/GeoIP/GeoLite2-ASN.mmdb"
    asnname = os.path.join(os.path.expanduser('~'), '.config/harpoon/asnnames.csv')
    asncidr = os.path.join(os.path.expanduser('~'), '.config/harpoon/asncidr.dat')
    specific_ips = os.path.join(os.path.expanduser('~'), '.config/harpoon/iplist.csv')

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('info', help='Information on an IP')
        parser_a.add_argument('IP', help='IP address')
        parser_a.set_defaults(subcommand='info')
        parser_b = subparsers.add_parser('intel', help='Gather Threat Intelligence information on an IP')
        parser_b.add_argument('IP', help='IP address')
        parser_b.set_defaults(subcommand='intel')
        self.parser = parser

    def update(self):
        file_name, headers = urllib.request.urlretrieve('http://www.cidr-report.org/as2.0/autnums.html')
        fin = open(file_name, 'r', encoding="latin-1", errors='ignore')
        fout = open(self.asnname, 'w+')
        line = fin.readline()
        reg = re.compile('^<a href="/cgi-bin/as-report\?as=AS\d+&view=2.0">AS(\d+)\s*</a> (.+)$')
        while line != '':
            res = reg.match(line)
            if res:
                fout.write('%s|%s\n' % (res.group(1), res.group(2)))
            line = fin.readline()
        fin.close()
        fout.close()
        print('-asnname.csv')
        print("Downloading CIDR data")
        try:
            os.remove(self.asncidr)
        except OSError:
            pass
        os.chdir("/tmp")
        subprocess.call(["pyasn_util_download.py", "--latest"])
        ls = glob.glob("rib*.bz2")[0]
        subprocess.call(['pyasn_util_convert.py', '--single', ls, 'latest.dat'])
        shutil.move('latest.dat', self.asncidr)
        print('-asncidr.dat')

    def ip_get_asn(self, ip):
        """
        Take an IP address and returns the asn number and name
        returns {'asn': 1234, 'name': 'FORTUM-AS Fortum, FI'}
        If not found, returns {'asn': 0, 'name': ''}
        """
        try:
            asndb = geoip2.database.Reader(self.geoasn)
            res = asndb.asn(ip)
            asn = res.autonomous_system_number
            asn_name = res.autonomous_system_organization
        except geoip2.errors.AddressNotFoundError:
            # TODO: check in the other ASN db
            return {'asn': 0, 'name': ''}
        return {'asn': asn, 'name': asn_name}

    def ipinfo(self, ip, dns=True):
        """
        Return information on an IP address
        {"asn", "asn_name", "city", "country"}
        """
        ipinfo = {}
        if dns:
            ipinfo['hostname'] = ''
            try:
                ipinfo['hostname'] = socket.gethostbyaddr(ip)[0]
            except socket.herror:
                pass
        try:
            citydb = geoip2.database.Reader(self.geocity)
            res = citydb.city(ip)
            ipinfo["city"] = res.city.name
            ipinfo["country"] = res.country.name
        except geoip2.errors.AddressNotFoundError:
            ipinfo["city"] = "Unknown"
            ipinfo["country"] = "Unknown"

        asninfo = self.ip_get_asn(ip)
        ipinfo['asn'] = asninfo['asn']
        ipinfo['asn_name'] = asninfo['name']
        ipinfo['specific'] = ''
        try:
            with open(self.specific_ips) as f:
                data = f.read().split('\n')
            for d in data:
                if d.strip().startswith(ip):
                    ipinfo['specific'] = d.split(',')[1].strip()
        except FileNotFoundError:
            pass
            # TODO: add private
        asnc = CommandAsn()
        res = asnc.asn_caida(ipinfo['asn'])
        ipinfo['asn_type'] = res['type']
        return ipinfo

    def run(self, conf, args, plugins):
        if 'subcommand' in args:
            if args.subcommand == 'info':
                if not is_ip(unbracket(args.IP)):
                    print("Invalid IP address")
                    sys.exit(1)
                # FIXME: move code here in a library
                ip = unbracket(args.IP)
                try:
                    ipy = IP(ip)
                except ValueError:
                    print('Invalid IP format, quitting...')
                    return
                ipinfo = self.ipinfo(ip)
                print('MaxMind: Located in %s, %s' % (ipinfo['city'], ipinfo['country']))
                if ipinfo['asn'] == 0:
                    print("MaxMind: IP not found in the ASN database")
                else:
                    print('MaxMind: ASN%i, %s' % (
                            ipinfo['asn'],
                            ipinfo['asn_name']
                        )
                    )
                    print('CAIDA Type: %s' % ipinfo['asn_type'])
                asndb2 = pyasn.pyasn(self.asncidr)
                res = asndb2.lookup(ip)
                if res[1] is None:
                    print("IP not found in ASN database")
                else:
                    # Search for name
                    f = open(self.asnname, 'r')
                    found = False
                    line = f.readline()
                    name = ''
                    while not found and line != '':
                        s = line.split('|')
                        if s[0] == str(res[0]):
                            name = s[1].strip()
                            found = True
                        line = f.readline()

                    print('ASN %i - %s (range %s)' % (
                            res[0],
                            name,
                            res[1]
                        )
                    )
                if ipinfo['hostname'] != '':
                    print('Hostname: %s' % ipinfo['hostname'])
                if ipinfo['specific'] != '':
                    print("Specific: %s" % ipinfo['specific'])
                if ipy.iptype() == "PRIVATE":
                    "Private IP"
                print("")
                if ipy.version() == 4:
                    print("Censys:\t\thttps://censys.io/ipv4/%s" % ip)
                    print("Shodan:\t\thttps://www.shodan.io/host/%s" % ip)
                    print("IP Info:\thttp://ipinfo.io/%s" % ip)
                    print("BGP HE:\t\thttps://bgp.he.net/ip/%s" % ip)
                    print("IP Location:\thttps://www.iplocation.net/?query=%s" % ip)
            elif args.subcommand == "intel":
                if not is_ip(unbracket(args.IP)):
                    print("Invalid IP address")
                    sys.exit(1)
                # Start with MISP and OTX to get Intelligence Reports
                print('###################### %s ###################' % unbracket(args.IP))
                passive_dns = []
                urls = []
                malware = []
                files = []
                # MISP
                misp_e = plugins['misp'].test_config(conf)
                if misp_e:
                    print('[+] Downloading MISP information...')
                    server = MispServer(url=conf['Misp']['url'], apikey=conf['Misp']['key'])
                    misp_results = server.attributes.search(value=unbracket(args.IP))

                # Binary Edge
                be_e = plugins['binaryedge'].test_config(conf)
                if be_e:
                    try:
                        print('[+] Downloading BinaryEdge information...')
                        be = BinaryEdge(conf['BinaryEdge']['key'])
                        # FIXME: this only get the first page
                        res = be.domain_ip(unbracket(args.IP))
                        for d in res["events"]:
                            passive_dns.append({
                                "domain": d['domain'],
                                "first": parse(d['updated_at']).astimezone(pytz.utc),
                                "last": parse(d['updated_at']).astimezone(pytz.utc),
                                "source" : "BinaryEdge"
                            })
                    except BinaryEdgeException:
                        print('BinaryEdge request failed, you need a paid subscription')
                # OTX
                otx_e = plugins['otx'].test_config(conf)
                if otx_e:
                    print('[+] Downloading OTX information....')
                    otx = OTXv2(conf["AlienVaultOtx"]["key"])
                    res = otx.get_indicator_details_full(IndicatorTypes.IPv4, unbracket(args.IP))
                    otx_pulses =  res["general"]["pulse_info"]["pulses"]
                    # Get Passive DNS
                    if "passive_dns" in res:
                        for r in res["passive_dns"]["passive_dns"]:
                            passive_dns.append({
                                "domain": r['hostname'],
                                "first": parse(r["first"]).astimezone(pytz.utc),
                                "last": parse(r["last"]).astimezone(pytz.utc),
                                "source" : "OTX"
                            })
                    if "url_list" in res:
                        for r in res["url_list"]["url_list"]:
                            urls.append(r)
                # RobTex
                print('[+] Downloading Robtex information....')
                rob = Robtex()
                try:
                    res = rob.get_ip_info(unbracket(args.IP))
                except RobtexError:
                    print("Error with Robtex")
                else:
                    for d in ["pas", "pash", "act", "acth"]:
                        if d in res:
                            for a in res[d]:
                                passive_dns.append({
                                    'first': a['date'].astimezone(pytz.utc),
                                    'last': a['date'].astimezone(pytz.utc),
                                    'domain': a['o'],
                                    'source': 'Robtex'
                                })

                # PT
                pt_e = plugins['pt'].test_config(conf)
                if pt_e:
                    out_pt = False
                    print('[+] Downloading Passive Total information....')
                    client = DnsRequest(conf['PassiveTotal']['username'], conf['PassiveTotal']['key'])
                    try:
                        raw_results = client.get_passive_dns(query=unbracket(args.IP))
                        if "results" in raw_results:
                            for res in raw_results["results"]:
                                passive_dns.append({
                                    "first": parse(res["firstSeen"]).astimezone(pytz.utc),
                                    "last": parse(res["lastSeen"]).astimezone(pytz.utc),
                                    "domain": res["resolve"],
                                    "source": "PT"
                                })
                        if "message" in raw_results:
                            if "quota_exceeded" in raw_results["message"]:
                                print("Quota exceeded for Passive Total")
                                out_pt = True
                                pt_osint = {}
                    except requests.exceptions.ReadTimeout:
                        print("Timeout on Passive Total requests")
                    if not out_pt:
                        try:
                            client2 = EnrichmentRequest(conf["PassiveTotal"]["username"], conf["PassiveTotal"]['key'])
                            # Get OSINT
                            # TODO: add PT projects here
                            pt_osint = client2.get_osint(query=unbracket(args.IP))
                            # Get malware
                            raw_results = client2.get_malware(query=unbracket(args.IP))
                            if "results" in raw_results:
                                for r in raw_results["results"]:
                                    malware.append({
                                        'hash': r["sample"],
                                        'date': parse(r['collectionDate']),
                                        'source' : 'PT (%s)' % r["source"]
                                    })
                        except requests.exceptions.ReadTimeout:
                            print("Timeout on Passive Total requests")
                # VT
                vt_e = plugins['vt'].test_config(conf)
                if vt_e:
                    if conf["VirusTotal"]["type"] != "public":
                        print('[+] Downloading VT information....')
                        vt = PrivateApi(conf["VirusTotal"]["key"])
                        res = vt.get_ip_report(unbracket(args.IP))
                        if "results" in res:
                            if "resolutions" in res['results']:
                                for r in res["results"]["resolutions"]:
                                    passive_dns.append({
                                        "first": parse(r["last_resolved"]).astimezone(pytz.utc),
                                        "last": parse(r["last_resolved"]).astimezone(pytz.utc),
                                        "domain": r["hostname"],
                                        "source": "VT"
                                    })
                            if "undetected_downloaded_samples" in res['results']:
                                for r in res['results']['undetected_downloaded_samples']:
                                    files.append({
                                        'hash': r['sha256'],
                                        'date': parse(r['date']),
                                        'source' : 'VT'
                                    })
                            if "undetected_referrer_samples" in res['results']:
                                for r in res['results']['undetected_referrer_samples']:
                                    if 'date' in r:
                                        files.append({
                                            'hash': r['sha256'],
                                            'date': parse(r['date']),
                                            'source' : 'VT'
                                        })
                                    else:
                                        #FIXME : should consider data without dates
                                        files.append({
                                            'hash': r['sha256'],
                                            'date': datetime.datetime(1970, 1, 1),
                                            'source' : 'VT'
                                        })
                            if "detected_downloaded_samples" in res['results']:
                                for r in res['results']['detected_downloaded_samples']:
                                    malware.append({
                                        'hash': r['sha256'],
                                        'date': parse(r['date']),
                                        'source' : 'VT'
                                    })
                            if "detected_referrer_samples" in res['results']:
                                for r in res['results']['detected_referrer_samples']:
                                    if "date" in r:
                                        malware.append({
                                            'hash': r['sha256'],
                                            'date': parse(r['date']),
                                            'source' : 'VT'
                                        })
                    else:
                        vt_e = False

                print('[+] Downloading GreyNoise information....')
                gn = GreyNoise()
                try:
                    greynoise = gn.query_ip(unbracket(args.IP))
                except GreyNoiseError:
                    greynoise = []

                tg_e = plugins['threatgrid'].test_config(conf)
                if tg_e:
                    print('[+] Downloading Threat Grid....')
                    try:
                        tg = ThreatGrid(conf['ThreatGrid']['key'])
                        res = tg.search_samples(unbracket(args.IP), type='ip')
                        already = []
                        if 'items' in res:
                            for r in res['items']:
                                if r['sample_sha256'] not in already:
                                    d = parse(r['ts'])
                                    d = d.replace(tzinfo=None)
                                    malware.append({
                                        'hash': r["sample_sha256"],
                                        'date': d,
                                        'source' : 'TG'
                                    })
                                    already.append(r['sample_sha256'])
                    except ThreatGridError as e:
                        print("Error with threat grid: {}".format(e.message))

                # ThreatMiner
                print('[+] Downloading ThreatMiner....')
                tm = ThreatMiner()
                response = tm.get_report(unbracket(args.IP))
                if response['status_code'] == '200':
                    tmm = response['results']
                else:
                    tmm = []
                    if response['status_code'] != '404':
                        print("Request to ThreatMiner failed: {}".format(response['status_message']))
                response = tm.get_related_samples(unbracket(args.IP))
                if response['status_code'] == '200':
                    for r in response['results']:
                        malware.append({
                            'hash': r,
                            'date': None,
                            'source': 'ThreatMiner'
                        })



                print('----------------- Intelligence Report')
                if otx_e:
                    if len(otx_pulses):
                        print('OTX:')
                        for p in otx_pulses:
                            print('- %s (%s - %s)' % (
                                    p['name'],
                                    p['created'][:10],
                                    "https://otx.alienvault.com/pulse/" + p['id']
                                )
                            )
                    else:
                        print('OTX: Not found in any pulse')
                if misp_e:
                    if len(misp_results) > 0:
                        print('MISP:')
                        for event in misp_results:
                            print("- %i - %s" % (event.id, event.info))
                if len(greynoise) > 0:
                    print("GreyNoise: IP identified as")
                    for r in greynoise:
                        print("\t%s (%s -> %s)" % (
                                r["name"],
                                r["first_seen"],
                                r["last_updated"]
                            )
                        )
                else:
                    print("GreyNoise: Not found")
                if pt_e:
                    if "results" in pt_osint:
                        if len(pt_osint["results"]):
                            if len(pt_osint["results"]) == 1:
                                if "name" in pt_osint["results"][0]:
                                    print("PT: %s %s" % (pt_osint["results"][0]["name"], pt_osint["results"][0]["sourceUrl"]))
                                else:
                                    print("PT: %s" % pt_osint["results"][0]["sourceUrl"])
                            else:
                                print("PT:")
                                for r in pt_osint["results"]:
                                    if "name" in r:
                                        print("-%s %s" % (r["name"], r["sourceUrl"]))
                                    else:
                                        print("-%s" % r["sourceUrl"])
                        else:
                            print("PT: Nothing found!")
                    else:
                        print("PT: Nothing found!")
                # ThreatMiner
                if len(tmm) > 0:
                    print("ThreatMiner:")
                    for r in tmm:
                        print("- {} {} - {}".format(
                            r['year'],
                            r['filename'],
                            r['URL']
                        ))

                if len(malware) > 0:
                    print('----------------- Malware')
                    for r in malware:
                        print("[%s] %s %s" % (
                                r["source"],
                                r["hash"],
                                r["date"].strftime("%Y-%m-%d") if r["date"] else ""
                            )
                        )
                if len(files) > 0:
                    print('----------------- Files')
                    for r in sorted(files, key=lambda x: x["date"]):
                        print("[%s] %s %s" % (
                                r["source"],
                                r["hash"],
                                r["date"].strftime("%Y-%m-%d")
                            )
                        )
                if len(passive_dns) > 0:
                    print('----------------- Passive DNS')
                    for r in sorted(passive_dns, key=lambda x: x["first"], reverse=True):
                        print("[+] %-40s (%s -> %s)(%s)" % (
                                r["domain"],
                                r["first"].strftime("%Y-%m-%d"),
                                r["last"].strftime("%Y-%m-%d"),
                                r["source"]
                            )
                        )


            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import operator
import json
from harpoon.commands.base import Command
from harpoon.lib.robtex import Robtex, RobtexError
from harpoon.lib.utils import json_serial, unbracket
from datetime import date, datetime

class CommandRobtex(Command):
    """
    # Robtex

    **Queries Robtex API https://www.robtex.com/api/**

    * Get information on an IP address: `harpoon robtex ip IP`
    * Get information on an ASN: `harpoon robtex asn 18342`
    * get information on a domain: `harpoon robtex domain example.org`

    The option `-j` shows raw JSON output.
    """
    name = "robtex"
    description = "Search in Robtex API (https://www.robtex.com/api/)"

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('ip', help='Request info on an IP')
        parser_a.add_argument('IP',  help='IP')
        parser_a.add_argument('--json', '-j', action='store_true', help='Show raw JSON info')
        parser_a.set_defaults(subcommand='ip')
        parser_b = subparsers.add_parser('asn', help='Request info on an ASN')
        parser_b.add_argument('ASN',  help='ASN', type=int)
        parser_b.add_argument('--json', '-j', action='store_true', help='Show raw JSON info')
        parser_b.set_defaults(subcommand='asn')
        parser_c = subparsers.add_parser('domain', help='Request info on a domain')
        parser_c.add_argument('DOMAIN',  help='DOMAIN')
        parser_c.add_argument('--json', '-j', action='store_true', help='Show raw JSON info')
        parser_c.set_defaults(subcommand='domain')
        self.parser = parser

    def run(self, conf, args, plugins):
        if 'subcommand' in args:
            if args.subcommand == "ip":
                r = Robtex()
                res = r.get_ip_info(args.IP)
                if args.json:
                    print(json.dumps(res, sort_keys=True, indent=4, default=json_serial))
                else:
                    print("AS %i: %s" % (res["as"], res["asname"]))
                    print("Location: %s, %s" % (res["city"], res["country"]))
                    print("BGP Route: %s, %s" % (res["bgproute"], res["routedesc"]))
                    print("Whois: %s" % res["whoisdesc"])
                    print("https://www.robtex.com/ip-lookup/%s" % args.IP)
                    if "pas" in res:
                        if len(res["pas"]):
                            print("Passive DNS:")
                            for d in res["pas"]:
                                print("\t%s %s" % (d["date"].isoformat(), d["o"]))
                    if "pash" in res:
                        if len(res["pash"]):
                            print("Passive DNS History:")
                            for d in res["pash"]:
                                print("\t%s %s" % (d["date"].isoformat(), d["o"]))
                    if "act" in res:
                        if len(res["act"]):
                            print("Active DNS:")
                            for d in res["act"]:
                                print("\t%s %s" % (d["date"].isoformat(), d["o"]))
                    if "acth" in res:
                        if len(res["acth"]):
                            print("ACtive DNS History:")
                            for d in res["acth"]:
                                print("\t%s %s" % (d["date"].isoformat(), d["o"]))
            elif args.subcommand == "asn":
                r = Robtex()
                res = r.get_asn_info(args.ASN)
                if args.json:
                    print(json.dumps(res, sort_keys=True, indent=4, default=json_serial))
                else:
                    print("ASN Routes:")
                    for n in res["nets"]:
                        print("[+] %s" % n["n"])
            elif args.subcommand == "domain":
                r = Robtex()
                res = r.get_pdns_domain(unbracket(args.DOMAIN))
                if args.json:
                    print(json.dumps(res, sort_keys=True, indent=4, default=json_serial))
                else:
                    if len(res) == 0:
                        print("No information on this domain")
                    else:
                        print("Passive DNS info:")
                        for r in res:
                            print("[+] %s\t%s\t(%s -> %s)" % (
                                    r["rrtype"],
                                    r["rrdata"],
                                    r["time_first_o"].isoformat(),
                                    r["time_last_o"].isoformat()
                                )
                            )
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import json
import requests
from harpoon.commands.base import Command
from harpoon.lib.urlscan import UrlScan


class CommandUrlscan(Command):
    """
    # url scan

    Allows to search and scan urls using https://urlscan.io/

    * Query the database : `harpoon urlscan search DOMAIN`
    * View an analysis : `harpoon urlscan view UID`
    """
    name = "urlscan"
    description = "Search and submit urls to urlscan.io"

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('search', help='Search in urlscan')
        parser_a.add_argument('QUERY', help='DOMAIN to be queried')
        parser_a.add_argument('--raw', '-r', action='store_true', help='Shows raw results')
        parser_a.set_defaults(subcommand='search')
        parser_c = subparsers.add_parser('view', help='View urlscan analysis')
        parser_c.add_argument('UID', help='UId of the analysis')
        parser_c.set_defaults(subcommand='view')
        self.parser = parser

    def run(self, conf, args, plugins):
        if 'subcommand' in args:
            us = UrlScan()
            if args.subcommand == 'search':
                # Search
                res = us.search(args.QUERY)
                if args.raw:
                    print(json.dumps(res, sort_keys=True, indent=4))
                else:
                    if len(res['results']) > 0:
                        for r in res['results']:
                            print("{} - {} - {} - https://urlscan.io/result/{}".format(
                                r["task"]["time"],
                                r["page"]["url"],
                                r["page"]["ip"],
                                r["_id"]
                                )
                            )
                    else:
                        print("No results for this query")
            elif args.subcommand == 'view':
                print(json.dumps(us.view(args.UID), sort_keys=True, indent=4))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import argparse
import configparser
import os
import sys
import json
from harpoon.commands.base import Command
from spyonweb import SpyOnWeb, SpyOnWebNotFound, SpyOnWebInvalidToken, SpyOnWebError


class CommandSpyonweb(Command):
    """
    # Spy On Web

    **Queries Spy On Web database http://spyonweb.com/**

    * Search for a domain: `harpoon spyonweb domain example.org`
    * Search for an adsense id: `harpoon spyonweb adsense ID`
    * Search for a Google Analytics id: `harpoon spyonweb analytics ID`
    * Search for an IP address: `harpoon spyonweb ip IP`
    """

    name = "spyonweb"
    description = "Search in SpyOnWeb through the API"
    config = {'SpyOnWeb': ['token']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommands')
        parser_b = subparsers.add_parser('domain', help='Query a domain')
        parser_b.add_argument('DOMAIN', help='Domain to be requested')
        parser_b.add_argument('--raw', '-r', action='store_true',
                help='Print raw list of domains')
        parser_b.set_defaults(subcommand='domain')
        parser_c = subparsers.add_parser('adsense', help='Query an adsense id')
        parser_c.add_argument('ID', help='id to be requested')
        parser_c.add_argument('--raw', '-r', action='store_true',
                help='Print raw list of domains')
        parser_c.set_defaults(subcommand='adsense')
        parser_d = subparsers.add_parser('analytics', help='Query a Google Analytics id')
        parser_d.add_argument('ID', help='id to be requested')
        parser_d.add_argument('--raw', '-r', action='store_true',
                help='Print raw list of domains')
        parser_d.set_defaults(subcommand='analytics')
        parser_e = subparsers.add_parser('ip', help='Query an IP Address')
        parser_e.add_argument('IP', help='IP address to be requested')
        parser_e.add_argument('--raw', '-r', action='store_true',
                help='Print raw list of domains')
        parser_e.set_defaults(subcommand='ip')
        parser_f = subparsers.add_parser('nsdomain', help='Query an Name Server domain')
        parser_f.add_argument('DOMAIN', help='Name Server Domain to be requested')
        parser_f.add_argument('--raw', '-r', action='store_true',
                help='Print raw list of domains')
        parser_f.set_defaults(subcommand='nsdomain')
        parser_g = subparsers.add_parser('nsip', help='Query a Name Server IP address')
        parser_g.add_argument('IP', help='Name Server IP Address to be requested')
        parser_g.add_argument('--raw', '-r', action='store_true',
                help='Print raw list of domains')
        parser_g.set_defaults(subcommand='nsip')
        self.parser = parser


    def run(self, conf, args, plugins):
        if hasattr(args, 'subcommand'):
            s = SpyOnWeb(conf['SpyOnWeb']['token'])
            if args.subcommand == 'domain':
                try:
                    res = s.summary(args.DOMAIN)
                except SpyOnWebNotFound:
                    print('Domain not found')
                except SpyOnWebInvalidToken:
                    print('Invalid configuration')
                except SpyOnWebError as e:
                    print('Weird error: %s' % e.message)
                else:
                    if args.raw:
                        print(json.dumps(res, sort_keys=True, indent=4))
                    else:
                        print('--------------- %s -----------------' % args.DOMAIN)
                        if 'ip' in res:
                            print('IP:')
                            for i in res['ip']:
                                print('-%s: %i entries' % (i, res['ip'][i]))
                        if 'adsense' in res:
                            print('AdSense:')
                            for i in res['adsense']:
                                print('-%s: %i entries' % (i, res['adsense'][i]))
                        if 'analytics' in res:
                            print('Analytics:')
                            for i in res['analytics']:
                                print('-%s: %i entries' % (i, res['analytics'][i]))
                        if 'dns_servers' in res:
                            print('DNS Servers:')
                            for i in res['dns_servers']:
                                print('-%s: %i entries' % (i, res['dns_servers'][i]))
            elif args.subcommand == 'adsense':
                try:
                    res = s.adsense(args.ID)
                except SpyOnWebNotFound:
                    print('Adsense id not found')
                except SpyOnWebInvalidToken:
                    print('Invalid configuration')
                except SpyOnWebError as e:
                    print('Weird error: %s' % e.message)
                else:
                    if args.raw:
                        for i in res['items']:
                            print(i)
                    else:
                        print('--------------- %s -----------------' % args.ID)
                        print('Fetched %i domains over %i' % (res['fetched'], res['found']))
                        for i in res['items']:
                            print('-%s (%s)' % (i, res['items'][i]))
            elif args.subcommand == 'analytics':
                try:
                    res = s.analytics(args.ID)
                except SpyOnWebNotFound:
                    print('Analytic id not found')
                except SpyOnWebInvalidToken:
                    print('Invalid configuration')
                except SpyOnWebError as e:
                    print('Weird error: %s' % e.message)
                else:
                    if args.raw:
                        for i in res['items']:
                            print(i)
                    else:
                        print('--------------- %s -----------------' % args.ID)
                        print('Fetched %i domains over %i' % (res['fetched'], res['found']))
                        for i in res['items']:
                            print('-%s (%s)' % (i, res['items'][i]))
            elif args.subcommand == 'ip':
                try:
                    res = s.ip(args.IP)
                except SpyOnWebNotFound:
                    print('IP address not found')
                except SpyOnWebInvalidToken:
                    print('Invalid configuration')
                except SpyOnWebError as e:
                    print('Weird error: %s' % e.message)
                else:
                    if args.raw:
                        for i in res['items']:
                            print(i)
                    else:
                        print('--------------- %s -----------------' % args.IP)
                        print('Fetched %i domains over %i' % (res['fetched'], res['found']))
                        for i in res['items']:
                            print('-%s (%s)' % (i, res['items'][i]))
            elif args.subcommand == 'nsdomain':
                try:
                    res = s.nameserver_domain(args.DOMAIN)
                except SpyOnWebNotFound:
                    print('Name Server domain not found')
                except SpyOnWebInvalidToken:
                    print('Invalid configuration')
                except SpyOnWebError as e:
                    print('Weird error: %s' % e.message)
                else:
                    if args.raw:
                        for i in res['items']:
                            print(i)
                    else:
                        print('--------------- %s -----------------' % args.DOMAIN)
                        print('Fetched %i domains over %i' % (res['fetched'], res['found']))
                        for i in res['items']:
                            print('-%s (%s)' % (i, res['items'][i]))
            elif args.subcommand == 'nsip':
                try:
                    res = s.nameserver_ip(args.IP)
                except SpyOnWebNotFound:
                    print('Name Server IP not found')
                except SpyOnWebInvalidToken:
                    print('Invalid configuration')
                except SpyOnWebError as e:
                    print('Weird error: %s' % e.message)
                else:
                    if args.raw:
                        for i in res['items']:
                            print(i)
                    else:
                        print('--------------- %s -----------------' % args.IP)
                        print('Fetched %i domains over %i' % (res['fetched'], res['found']))
                        for i in res['items']:
                            print('-%s (%s)' % (i, res['items'][i]))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import json
from harpoon.commands.base import Command
from pyhunter import PyHunter

class CommandHunter(Command):
    """
    # Hunter.io

    **Requests Hunter.io plugin https://hunter.io/**

    * Search for an email: `harpoon hunter email FIRSTNAME LASTNAME DOMAIN`
    * Search for a domain: `harpoon hunder domain example.org`
    """
    name = "hunter"
    description = "Request hunter.io information through the API"
    config = { 'Hunter': ['key']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('email')
        parser_a.add_argument('NAME', help='Name of the user')
        parser_a.add_argument('DOMAIN', help='Domain of the user')
        parser_a.set_defaults(subcommand='email')
        parser_a = subparsers.add_parser('domain')
        parser_a.add_argument('DOMAIN', help='Domain to look for')
        parser_a.set_defaults(subcommand='domain')
        self.parser = parser

    def run(self, conf, args, plugins):
        if 'subcommand' in args:
            hunter = PyHunter(conf['Hunter']['key'])
            if args.subcommand == 'email':
                if ' ' not in args.NAME:
                    print('Name should contains first and last name')
                    print('(Yes this API is useless)')
                    sys.exit(1)
                res = hunter.email_finder(
                    domain=args.DOMAIN,
                    full_name=args.NAME,
                    raw=True
                )
                print(json.dumps(res, sort_keys=True, indent=4))
            elif args.subcommand == 'domain':
                res = hunter.domain_search(args.DOMAIN)
                print(json.dumps(res, sort_keys=True, indent=4))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import requests
import os
import json
import datetime
import urllib.request
import tarfile
import geoip2.database
import re
import subprocess
import glob
import shutil
import pyasn
import pypdns
import pytz
from IPy import IP
from dateutil.parser import parse
from harpoon.commands.base import Command
from harpoon.lib.utils import bracket, unbracket
from harpoon.lib.robtex import Robtex, RobtexError
from harpoon.lib.urlscan import UrlScan
from OTXv2 import OTXv2, IndicatorTypes
from virus_total_apis import PublicApi, PrivateApi
from pygreynoisev1 import GreyNoise, GreyNoiseError
from passivetotal.libs.dns import DnsRequest
from passivetotal.libs.enrichment import EnrichmentRequest
from pythreatgrid2 import ThreatGrid, ThreatGridError
from pybinaryedge import BinaryEdge, BinaryEdgeException, BinaryEdgeNotFound
from threatminer import ThreatMiner


class CommandDomain(Command):
    """
    # Domain

    """
    name = "domain"
    description = "Gather information on a domain"
    config = None
    geocity = os.path.join(os.path.expanduser('~'), '.config/harpoon/GeoLite2-City.mmdb')
    geoasn = os.path.join(os.path.expanduser('~'), '.config/harpoon/GeoLite2-ASN.mmdb')
    asnname = os.path.join(os.path.expanduser('~'), '.config/harpoon/asnnames.csv')
    asncidr = os.path.join(os.path.expanduser('~'), '.config/harpoon/asncidr.dat')

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('info', help='Information on an domain')
        parser_a.add_argument('DOMAIN', help='Domain')
        parser_a.set_defaults(subcommand='info')
        parser_b = subparsers.add_parser('intel', help='Gather Threat Intelligence information on a domain')
        parser_b.add_argument('DOMAIN', help='Domain')
        parser_b.set_defaults(subcommand='intel')
        self.parser = parser

    def ipinfo(self, ip):
        """
        Return information on an IP address
        {"asn", "asn_name", "city", "country"}
        """
        ipinfo = {}
        try:
            citydb = geoip2.database.Reader(self.geocity)
            res = citydb.city(ip)
            ipinfo["city"] = res.city.name
            ipinfo["country"] = res.country.name
        except geoip2.errors.AddressNotFoundError:
            ipinfo["city"] = "Unknown"
            ipinfo["country"] = "Unknown"
        try:
            asndb = geoip2.database.Reader(self.geoasn)
            res = asndb.asn(ip)
            ipinfo["asn"] = res.autonomous_system_number
            ipinfo["asn_name"] = res.autonomous_system_organization
        except geoip2.errors.AddressNotFoundError:
            ipinfo["asn"] = ""
            ipinfo["asn_name"] = ""
            # FIXME: check in text files if not found
            # TODO: add private
        return ipinfo

    def run(self, conf, args, plugins):
        if 'subcommand' in args:
            if args.subcommand == 'info':
                print("Not implemented yet")
            elif args.subcommand == "intel":
                # Start with MISP and OTX to get Intelligence Reports
                print('###################### %s ###################' % args.DOMAIN)
                passive_dns = []
                urls = []
                malware = []
                files = []
                # OTX
                otx_e = plugins['otx'].test_config(conf)
                if otx_e:
                    print('[+] Downloading OTX information....')
                    try:
                        otx = OTXv2(conf["AlienVaultOtx"]["key"])
                        res = otx.get_indicator_details_full(IndicatorTypes.DOMAIN, unbracket(args.DOMAIN))
                        otx_pulses =  res["general"]["pulse_info"]["pulses"]
                        # Get Passive DNS
                        if "passive_dns" in res:
                            for r in res["passive_dns"]["passive_dns"]:
                                passive_dns.append({
                                    "ip": r['hostname'],
                                    "first": parse(r["first"]).astimezone(pytz.utc),
                                    "last": parse(r["last"]).astimezone(pytz.utc),
                                    "source" : "OTX"
                                })
                        if "url_list" in res:
                            for r in res["url_list"]["url_list"]:
                                if "result" in r:
                                    urls.append({
                                        "date": parse(r["date"]).astimezone(pytz.utc),
                                        "url": r["url"],
                                        "ip": r["result"]["urlworker"]["ip"] if "ip" in r["result"]["urlworker"] else "" ,
                                        "source": "OTX"
                                    })
                                else:
                                    urls.append({
                                        "date": parse(r["date"]).astimezone(pytz.utc),
                                        "url": r["url"],
                                        "ip": "",
                                        "source": "OTX"
                                    })
                    except AttributeError:
                        print('OTX crashed  ¯\_(ツ)_/¯')

                # UrlScan
                us = UrlScan()
                print('[+] Downloading UrlScan information....')
                res = us.search(args.DOMAIN)
                for r in res['results']:
                    urls.append({
                        "date": parse(r["task"]["time"]).astimezone(pytz.utc),
                        "url": r["page"]["url"],
                        "ip": r["page"]["ip"] if "ip" in r["page"] else "",
                        "source": "UrlScan"
                    })

                # CIRCL
                circl_e = plugins['circl'].test_config(conf)
                if circl_e:
                    print('[+] Downloading CIRCL passive DNS information....')
                    x = pypdns.PyPDNS(
                        basic_auth=(
                            conf['Circl']['user'],
                            conf['Circl']['pass']
                        )
                    )
                    res = x.query(unbracket(args.DOMAIN))
                    for answer in res:
                        passive_dns.append({
                            "ip": answer['rdata'],
                            "first": answer['time_first'].astimezone(pytz.utc),
                            "last": answer['time_last'].astimezone(pytz.utc),
                            "source" : "CIRCL"
                        })
                # BinaryEdge
                be_e = plugins['binaryedge'].test_config(conf)
                if be_e:
                    print('[+] Downloading BinaryEdge information....')
                    try:
                        be = BinaryEdge(conf['BinaryEdge']['key'])
                        res = be.domain_dns(unbracket(args.DOMAIN))
                        for d in res['events']:
                            if "A" in d:
                                for a in d['A']:
                                    passive_dns.append({
                                        "ip": a,
                                        "first": parse(d['updated_at']).astimezone(pytz.utc),
                                        "last": parse(d['updated_at']).astimezone(pytz.utc),
                                        "source" : "BinaryEdge"
                                    })
                    except BinaryEdgeException:
                        print('You need a paid BinaryEdge subscription for this request')
                # RobTex
                print('[+] Downloading Robtex information....')
                try:
                    rob = Robtex()
                    res = rob.get_pdns_domain(args.DOMAIN)
                    for d in res:
                        if d['rrtype'] in ['A', 'AAAA']:
                            passive_dns.append({
                                'first': d['time_first_o'].astimezone(pytz.utc),
                                'last': d['time_last_o'].astimezone(pytz.utc),
                                'ip': d['rrdata'],
                                'source': 'Robtex'
                            })
                except RobtexError:
                    print("Robtex query failed")
                # PT
                pt_e = plugins['pt'].test_config(conf)
                if pt_e:
                    try:
                        pt_osint = {}
                        ptout = False
                        print('[+] Downloading Passive Total information....')
                        client = DnsRequest(conf['PassiveTotal']['username'], conf['PassiveTotal']['key'])
                        raw_results = client.get_passive_dns(query=unbracket(args.DOMAIN))
                        if "results" in raw_results:
                            for res in raw_results["results"]:
                                passive_dns.append({
                                    "first": parse(res["firstSeen"]).astimezone(pytz.utc),
                                    "last": parse(res["lastSeen"]).astimezone(pytz.utc),
                                    "ip": res["resolve"],
                                    "source": "PT"
                                })
                        if "message" in raw_results:
                            if "quota_exceeded" in raw_results["message"]:
                                print("PT quota exceeded")
                                ptout = True
                        if not ptout:
                            client2 = EnrichmentRequest(conf["PassiveTotal"]["username"], conf["PassiveTotal"]['key'])
                            # Get OSINT
                            # TODO: add PT projects here
                            pt_osint = client2.get_osint(query=unbracket(args.DOMAIN))
                            # Get malware
                            raw_results = client2.get_malware(query=unbracket(args.DOMAIN))
                            if "results" in raw_results:
                                for r in raw_results["results"]:
                                    malware.append({
                                        'hash': r["sample"],
                                        'date': parse(r['collectionDate']).astimezone(pytz.utc),
                                        'source' : 'PT (%s)' % r["source"]
                                    })
                    except requests.exceptions.ReadTimeout:
                        print("PT: Time Out")
                # VT
                vt_e = plugins['vt'].test_config(conf)
                if vt_e:
                    if conf["VirusTotal"]["type"] != "public":
                        print('[+] Downloading VT information....')
                        vt = PrivateApi(conf["VirusTotal"]["key"])
                        res = vt.get_domain_report(unbracket(args.DOMAIN))
                        if "results" in res:
                            if "resolutions" in res['results']:
                                for r in res["results"]["resolutions"]:
                                    passive_dns.append({
                                        "first": parse(r["last_resolved"]).astimezone(pytz.utc),
                                        "last": parse(r["last_resolved"]).astimezone(pytz.utc),
                                        "ip": r["ip_address"],
                                        "source": "VT"
                                    })
                            if "undetected_downloaded_samples" in res['results']:
                                for r in res['results']['undetected_downloaded_samples']:
                                    files.append({
                                        'hash': r['sha256'],
                                        'date': parse(r['date']).astimezone(pytz.utc) if 'date' in r else '',
                                        'source' : 'VT'
                                    })
                            if "undetected_referrer_samples" in res['results']:
                                for r in res['results']['undetected_referrer_samples']:
                                    files.append({
                                        'hash': r['sha256'],
                                        'date': parse(r['date']).astimezone(pytz.utc) if 'date' in r else '',
                                        'source' : 'VT'
                                    })
                            if "detected_downloaded_samples" in res['results']:
                                for r in res['results']['detected_downloaded_samples']:
                                    malware.append({
                                        'hash': r['sha256'],
                                        'date': parse(r['date']).astimezone(pytz.utc),
                                        'source' : 'VT'
                                    })
                            if "detected_referrer_samples" in res['results']:
                                for r in res['results']['detected_referrer_samples']:
                                    if "date" in r:
                                        malware.append({
                                            'hash': r['sha256'],
                                            'date': parse(r['date']).astimezone(pytz.utc),
                                            'source' : 'VT'
                                        })
                            if "detected_urls" in res['results']:
                                for r in res['results']['detected_urls']:
                                    urls.append({
                                        'date': parse(r['scan_date']).astimezone(pytz.utc),
                                        'url': r['url'],
                                        'ip': '',
                                        'source': 'VT'
                                    })
                    else:
                        vt_e = False
                tg_e = plugins['threatgrid'].test_config(conf)
                if tg_e:
                    try:
                        print('[+] Downloading Threat Grid....')
                        tg = ThreatGrid(conf['ThreatGrid']['key'])
                        res = tg.search_samples(unbracket(args.DOMAIN), type='domain')
                        already = []
                        if 'items' in res:
                            for r in res['items']:
                                if r['sample_sha256'] not in already:
                                    d = parse(r['ts']).astimezone(pytz.utc)
                                    malware.append({
                                        'hash': r["sample_sha256"],
                                        'date': d,
                                        'source' : 'ThreatGrid'
                                    })
                                    already.append(r['sample_sha256'])
                    except ThreatGridError as e:
                        print("Failed to connect to Threat Grid: %s" % e.message)
                print('[+] Downloading ThreatMiner....')
                tm = ThreatMiner()
                response = tm.get_report(unbracket(args.DOMAIN))
                if response['status_code'] == '200':
                    tmm = response['results']
                else:
                    tmm = []
                    if response['status_code'] == '404':
                        print("Request to ThreatMiner failed: {}".format(response['status_message']))
                response = tm.get_related_samples(unbracket(args.DOMAIN))
                if response['status_code'] == '200':
                    for r in response['results']:
                        malware.append({
                            'hash': r,
                            'date': None,
                            'source': 'ThreatMiner'
                        })


                # TODO: Add MISP
                print('----------------- Intelligence Report')
                if otx_e:
                    if len(otx_pulses):
                        print('OTX:')
                        for p in otx_pulses:
                            print('- %s (%s - %s)' % (
                                    p['name'],
                                    p['created'][:10],
                                    "https://otx.alienvault.com/pulse/" + p['id']
                                )
                            )
                    else:
                        print('OTX: Not found in any pulse')
                if pt_e:
                    if "results" in pt_osint:
                        if len(pt_osint["results"]):
                            if len(pt_osint["results"]) == 1:
                                if "name" in pt_osint["results"][0]:
                                    print("PT: %s %s" % (pt_osint["results"][0]["name"], pt_osint["results"][0]["sourceUrl"]))
                                else:
                                    print("PT: %s" % (pt_osint["results"][0]["sourceUrl"]))
                            else:
                                print("PT:")
                                for r in pt_osint["results"]:
                                    if "name" in r:
                                        print("- %s %s" % (r["name"], r["sourceUrl"]))
                                    else:
                                        print("- %s" % (r["sourceUrl"]))
                        else:
                            print("PT: Nothing found!")
                    else:
                        print("PT: Nothing found!")
                # ThreatMiner
                if len(tmm) > 0:
                    print("ThreatMiner:")
                    for r in tmm:
                        print("- {} {} - {}".format(
                            r['year'],
                            r['filename'],
                            r['URL']
                        ))

                if len(malware) > 0:
                    print('----------------- Malware')
                    for r in malware:
                        print("[%s] %s %s" % (
                                r["source"],
                                r["hash"],
                                r["date"].strftime("%Y-%m-%d") if r["date"] else ""
                            )
                        )
                if len(files) > 0:
                    print('----------------- Files')
                    for r in files:
                        if r['date'] != '':
                            print("[%s] %s (%s)" % (
                                    r["source"],
                                    r["hash"],
                                    r["date"].strftime("%Y-%m-%d")
                                )
                            )
                        else:
                            print("[%s] %s" % (
                                    r["source"],
                                    r["hash"],
                                )
                            )
                if len(urls) > 0:
                    print('----------------- Urls')
                    for r in sorted(urls, key=lambda x: x["date"], reverse=True):
                        print("[%s] %s - %s %s" % (
                                r["source"],
                                r["url"],
                                r["ip"],
                                r["date"].strftime("%Y-%m-%d")
                            )
                        )
                # TODO: add ASN + location info here
                if len(passive_dns) > 0:
                    print('----------------- Passive DNS')
                    for r in sorted(passive_dns, key=lambda x: x["first"], reverse=True):
                        print("[+] %-40s (%s -> %s)(%s)" % (
                                r["ip"],
                                r["first"].strftime("%Y-%m-%d"),
                                r["last"].strftime("%Y-%m-%d"),
                                r["source"]
                            )
                        )

            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import json
from harpoon.lib.utils import unbracket
from harpoon.commands.base import Command
from pybinaryedge import BinaryEdge, BinaryEdgeException, BinaryEdgeNotFound


class CommandBitly(Command):
    """
    # binaryedge plugin

    **Query the BinaryEdge API https://www.binaryedge.io/**

    * Check details on an IP: `harpoon binaryedge ip IP`
    * Search for a specific query: `harpoon binaryedge search QUERY`
    """
    name = "binaryedge"
    description = "Request BinaryEdge API"
    config = {'BinaryEdge': ['key']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Commands')
        parser_a = subparsers.add_parser('config', help='Configure pybinary edge')
        parser_a.add_argument('--key', '-k', help='Configure the API key')
        parser_a.set_defaults(which='config')
        parser_b = subparsers.add_parser('ip', help='Query an IP address')
        parser_b.add_argument('IP', help='IP to be requested')
        parser_b.add_argument(
            '--historical', '-H', action='store_true',
            help='Requests historical data about an IP'
        )
        parser_b.add_argument(
            '--score', '-s', action='store_true',
            help='Requests vulnerability score about an IP'
        )
        parser_b.add_argument(
            '--image', '-i', action='store_true',
            help='Requests images identified for an IP'
        )
        parser_b.add_argument(
            '--torrent', '-t', action='store_true',
            help='Request torrents identified for an IP'
        )
        parser_b.add_argument(
            '--dns', '-d', action='store_true',
            help='Requests images identified for an IP'
        )
        parser_b.add_argument('--page', '-p', type=int, default=1,
            help='Get specific page')
        parser_b.set_defaults(which='ip')
        parser_c = subparsers.add_parser('search', help='Search in the database')
        parser_c.add_argument('SEARCH', help='Search request')
        parser_c.add_argument(
            '--page', '-p', type=int, default=1,
            help='Get specific page'
        )
        parser_c.add_argument(
            '--image', '-i', action='store_true',
            help='Requests images identified for an IP'
        )
        parser_c.set_defaults(which='search')
        parser_d = subparsers.add_parser('dataleaks', help='Search in the leaks database')
        parser_d.add_argument('EMAIL', help='Search email in the leaks database')
        parser_d.add_argument(
            '--domain', '-d', action='store_true',
            help='Search for domain instead of email'
        )
        parser_d.set_defaults(which='dataleaks')
        parser_e = subparsers.add_parser('domains', help='Search information on a domain')
        parser_e.add_argument('DOMAIN', help='Domain to be requested')
        parser_e.add_argument('--page', '-p', type=int, default=1,
                help='Get specific page')
        parser_e.add_argument(
            '--subdomains', '-s', action='store_true',
            help='Returns subdomains'
        )
        parser_e.set_defaults(which='domain')
        self.parser = parser

    def run(self, conf, args, plugins):
        be = BinaryEdge(conf['BinaryEdge']['key'])
        try:
            if hasattr(args, 'which'):
                if args.which == 'ip':
                    if args.score:
                        res = be.host_score(unbracket(args.IP))
                    elif args.image:
                        res = be.image_ip(unbracket(args.IP))
                    elif args.torrent:
                        if args.historical:
                            res = be.torrent_historical_ip(unbracket(args.IP))
                        else:
                            res = be.torrent_ip(unbracket(args.IP))
                    elif args.historical:
                        res = be.host_historical(unbracket(args.IP))
                    elif args.dns:
                        res = be.domain_ip(args.IP, page=args.page)
                    else:
                        res = be.host(unbracket(args.IP))
                    print(json.dumps(res, sort_keys=True, indent=4))
                elif args.which == 'search':
                    if args.image:
                        res = be.image_search(args.SEARCH, page=args.page)
                    else:
                        res = be.host_search(args.SEARCH, page=args.page)
                    print(json.dumps(res, sort_keys=True, indent=4))
                elif args.which == 'dataleaks':
                    if args.domain:
                        res = be.dataleaks_organization(args.EMAIL)
                    else:
                        res = be.dataleaks_email(args.EMAIL)
                    print(json.dumps(res, sort_keys=True, indent=4))
                elif args.which == 'domain':
                    if args.subdomains:
                        res = be.domain_subdomains(args.DOMAIN, page=args.page)
                    else:
                        res = be.domain_dns(args.DOMAIN, page=args.page)
                    print(json.dumps(res, sort_keys=True, indent=4))
                else:
                    self.parser.print_help()
            else:
                self.parser.print_help()
        except ValueError as e:
            print('Invalid Value: %s' % e.message)
        except BinaryEdgeNotFound:
            print('Search term not found')
        except BinaryEdgeException as e:
            print('Error: %s' % e.message)



#! /usr/bin/env python
import sys
import json
import hashlib
import os
from harpoon.commands.base import Command
from virus_total_apis import PublicApi, PrivateApi
from harpoon.lib.utils import unbracket


class CommandVirusTotal(Command):
    """
    # Virus Total

    * Search for hash: `harpoon vt hash HASH`
    * Search for a domain: `harpoon vt domain example.org`
    * Search for an IP: `harpoon vt ip IP`
    * Check a file in VT: `harpoon vt file FILE` (check for the hash, no upload)
    * Download a file (private only): `harpoon vt dl HASH`
    * Search for a list of hashes from a file: `harpoon vt hashlist FILE`
    * Search for a list of domains from a file : `harpoon vt domainlist FILE`
    * Search for a list of IP addresses from a file : `harpoon vt iplist FILE`
    """
    name = "vt"
    description = "Request Virus Total API"
    config = {'VirusTotal': ['key', 'type']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('hash', help='Request info on a hash')
        parser_a.add_argument('HASH', help='Hash')
        parser_a.add_argument('--raw', '-r', help='Raw data', action='store_true')
        parser_a.add_argument('--extended', '-e', help='Extended info (private API only)', action='store_true')
        parser_a.set_defaults(subcommand='hash')
        parser_b = subparsers.add_parser('hashlist', help='Request a list of hashes')
        parser_b.add_argument('FILE',  help='File containing the domains')
        parser_b.set_defaults(subcommand='hashlist')
        parser_c = subparsers.add_parser('domain', help='Request info on a domain')
        parser_c.add_argument('DOMAIN',  help='Domain')
        parser_c.add_argument('--json', '-j', action='store_true', help='Show raw JSON info')
        parser_c.set_defaults(subcommand='domain')
        parser_d = subparsers.add_parser('ip', help='Request info on an IP')
        parser_d.add_argument('IP',  help='IP')
        parser_d.set_defaults(subcommand='ip')
        parser_e = subparsers.add_parser('url', help='Request info on an URL')
        parser_e.add_argument('URL',  help='URL')
        parser_e.set_defaults(subcommand='url')
        parser_f = subparsers.add_parser('domainlist', help='Request info on a list of domains')
        parser_f.add_argument('FILE',  help='File containing the list of domains')
        parser_f.set_defaults(subcommand='domainlist')
        parser_e = subparsers.add_parser('file', help='Request info on a file (no upload)')
        parser_e.add_argument('FILE', help='File')
        parser_e.add_argument('--raw', '-r', help='Raw data', action='store_true')
        parser_e.set_defaults(subcommand='file')
        parser_f = subparsers.add_parser('dl', help='Download a file from VT')
        parser_f.add_argument('HASH', help='Hash of the file')
        parser_f.set_defaults(subcommand='dl')
        parser_g = subparsers.add_parser('iplist', help='Request info on a list of IP addresses')
        parser_g.add_argument('FILE',  help='File containing the list of IPs')
        parser_g.set_defaults(subcommand='iplist')
        self.parser = parser

    def print_domaininfo(self, res):
        """Print nicely the domain information"""
        if "results" in res:
            if "verbose_msg" in res["results"]:
                print(res["results"]["verbose_msg"])
            if "detected_urls" in res["results"]:
                if len(res["results"]["detected_urls"]) > 0:
                    print("-Detected urls:")
                    for r in res["results"]["detected_urls"]:
                        print("\t%s (on %s, %i/%i)" % (
                                r["url"],
                                r["scan_date"],
                                r["positives"],
                                r["total"]
                            )
                        )
            if "undetected_urls" in res["results"]:
                print("-Undetected urls:")
                if len(res["results"]["undetected_urls"]) > 0:
                    for r in res["results"]["undetected_urls"]:
                        print("\t%s (on %s, %i/%i)" % (
                                r[0],
                                r[4],
                                r[2],
                                r[3]
                            )
                        )
            if "resolutions" in res["results"]:
                if len(res["results"]["resolutions"]) > 0:
                    print("-Resolutions:")
                    for r in res["results"]["resolutions"]:
                        print("\t%s (%s)" % (r["ip_address"], r["last_resolved"]))
            if "detected_referrer_samples" in res["results"]:
                if len(res["results"]["detected_referrer_samples"]) > 0:
                    print("-Detected Referrer Sample:")
                    for r in res["results"]["detected_referrer_samples"]:
                        print("\t%s (%i/%i)" % (
                                r["sha256"],
                                r["positives"],
                                r["total"]
                            )
                        )
            if "undetected_referrer_samples" in res["results"]:
                if len(res["results"]["undetected_referrer_samples"]) > 0:
                    print("-Undetected Referrer Sample:")
                    for r in res["results"]["undetected_referrer_samples"]:
                        print("\t%s (%i/%i)" % (
                                r["sha256"],
                                r["positives"],
                                r["total"]
                            )
                        )
            if "undetected_downloaded_samples" in res["results"]:
                if len(res["results"]["undetected_downloaded_samples"]) > 0:
                    print("-Undetected Downloaded Sample:")
                    for r in res["results"]["undetected_downloaded_samples"]:
                        print("\t%s (on %s, %i/%i)" % (
                                r["sha256"],
                                r["date"],
                                r["positives"],
                                r["total"]
                            )
                        )
            if "detected_downloaded_samples" in res["results"]:
                if len(res["results"]["detected_downloaded_samples"]) > 0:
                    print("-Detected Downloaded Sample:")
                    for r in res["results"]["detected_downloaded_samples"]:
                        print("\t%s (on %s, %i/%i)" % (
                                r["sha256"],
                                r["date"],
                                r["positives"],
                                r["total"]
                            )
                        )

    def print_file(self, response):
        """
        Print details on a file
        """
        if response["response_code"] != 200:
            print("Error with the request (reponse code %i)" % response["response_code"])
            sys.exit(1)
        if response["results"]["response_code"] == 0:
            print("File not found")
            sys.exit(0)
        print("[+] Detection: %i / %i" % (
                response["results"]["positives"],
                response["results"]["total"]
            )
        )
        print("[+] MD5: %s" % response["results"]["md5"])
        print("[+] SHA1: %s" % response["results"]["sha1"])
        print("[+] SHA256: %s" % response["results"]["sha256"])
        if "first_seen" in response['results']:
            print("[+] First Seen: %s" % response["results"]["first_seen"])
        if "last_seen" in response['results']:
            print("[+] Last Seen: %s" % response["results"]["last_seen"])
        print("[+] Link: %s" % response["results"]["permalink"])

    def run(self, conf, args, plugins):
        if 'subcommand' in args:
            if conf["VirusTotal"]["type"] != "public":
                vt = PrivateApi(conf["VirusTotal"]["key"])
                if args.subcommand == "hash":
                    response = vt.get_file_report(args.HASH)
                    if args.raw:
                        print(json.dumps(response, sort_keys=False, indent=4))
                        if args.extended:
                            response = vt.get_network_traffic(args.HASH)
                            print(json.dumps(response, sort_keys=False, indent=4))
                            response = vt.get_file_behaviour(args.HASH)
                            print(json.dumps(response, sort_keys=False, indent=4))
                    else:
                        self.print_file(response)
                elif args.subcommand == "dl":
                    if os.path.isfile(args.HASH):
                        print("File %s already exists" % args.HASH)
                        sys.exit(0)
                    data = vt.get_file(args.HASH)
                    if isinstance(data, dict):
                        if 'results' in data:
                            with open(args.HASH, "wb") as f:
                                f.write(data['results'])
                            print("File downloaded as %s" % args.HASH)
                        else:
                            print('Invalid answer format')
                            sys.exit(1)
                    else:
                        with open(args.HASH, "wb") as f:
                            f.write(data)
                        print("File downloaded as %s" % args.HASH)

                elif args.subcommand == "file":
                    with open(args.FILE, "rb") as f:
                        # FIXME : could be more efficient
                        data = f.read()
                    m = hashlib.sha256()
                    m.update(data)
                    h = m.hexdigest()
                    response = vt.get_file_report(h)
                    if args.raw:
                        print(json.dumps(response, sort_keys=False, indent=4))
                    else:
                        self.print_file(response)
                elif args.subcommand == "hashlist":
                    with open(args.FILE, 'r') as infile:
                        data = infile.read().split()
                    hash_list = list(set([a.strip() for a in data]))
                    print("Hash;Found;Detection;Total AV;First Seen;Last Seen;Link")
                    for h in hash_list:
                        response = vt.get_file_report(h)
                        if response["response_code"] != 200:
                            print("Error with the request (reponse code %i)" % response["response_code"])
                            print(json.dumps(response, sort_keys=False, indent=4))
                            print("Quitting...")
                            sys.exit(1)
                        if "response_code" in response["results"]:
                            if response["results"]["response_code"] == 0:
                                print("%s;Not found;;;;;" % h)
                            else:
                                print("%s;Found;%i;%i;%s;%s;%s" % (
                                        h,
                                        response["results"]["positives"],
                                        response["results"]["total"],
                                        response["results"]["first_seen"],
                                        response["results"]["last_seen"],
                                        response["results"]["permalink"]
                                    )
                                )
                        else:
                            print("%s;Not found;;;;;" % h)
                elif args.subcommand == "domainlist":
                    with open(args.FILE, 'r') as infile:
                        data = infile.read().split()
                    for d in data:
                        print("################ Domain %s" % d.strip())
                        res = vt.get_domain_report(d.strip())
                        self.print_domaininfo(res)
                elif args.subcommand == "iplist":
                    with open(args.FILE, 'r') as infile:
                        data = infile.read().split()
                    for d in data:
                        print("################ IP %s" % d.strip())
                        res = vt.get_ip_report(unbracket(d.strip()))
                        print(json.dumps(res, sort_keys=False, indent=4))
                elif args.subcommand == "domain":
                    res = vt.get_domain_report(unbracket(args.DOMAIN))
                    if args.json:
                        print(json.dumps(res, sort_keys=False, indent=4))
                    else:
                        self.print_domaininfo(res)
                elif args.subcommand == "ip":
                    res = vt.get_ip_report(unbracket(args.IP))
                    print(json.dumps(res, sort_keys=False, indent=4))
                elif args.subcommand == "url":
                    res = vt.get_url_report(args.URL)
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    self.parser.print_help()
            else:
                vt = PublicApi(conf["VirusTotal"]["key"])
                if args.subcommand == "hash":
                    response = vt.get_file_report(args.HASH)
                    if args.raw:
                        print(json.dumps(response, sort_keys=False, indent=4))
                    else:
                        self.print_file(response)
                elif args.subcommand == "file":
                    with open(args.FILE, "rb") as f:
                        # FIXME : could be more efficient
                        data = f.read()
                    m = hashlib.sha256()
                    m.update(data)
                    response = vt.get_file_report(m.hexdigest())
                    if args.raw:
                        print(json.dumps(response, sort_keys=False, indent=4))
                    else:
                        self.print_file(response)
                elif args.subcommand == "hashlist":
                    with open(args.FILE, 'r') as infile:
                        data = infile.read().split()
                    hash_list = list(set([a.strip() for a in data]))
                    print("Hash;Found;Detection;Total AV;Link")
                    for h in hash_list:
                        response = vt.get_file_report(h)
                        if response["response_code"] != 200:
                            print("Error with the request (reponse code %i)" % response["response_code"])
                            print(json.dumps(response, sort_keys=False, indent=4))
                            print("Quitting...")
                            sys.exit(1)
                        if "response_code" in response["results"]:
                            if response["results"]["response_code"] == 0:
                                print("%s;Not found;;;" % h)
                            else:
                                print("%s;Found;%i;%i;%s" % (
                                        h,
                                        response["results"]["positives"],
                                        response["results"]["total"],
                                        response["results"]["permalink"]
                                    )
                                )
                        else:
                            print("%s;Not found;;;" % h)
                elif args.subcommand == "domain":
                    res = vt.get_domain_report(unbracket(args.DOMAIN))
                    if args.json:
                        print(json.dumps(res, sort_keys=False, indent=4))
                    else:
                        self.print_domaininfo(res)
                elif args.subcommand == "ip":
                    res = vt.get_ip_report(unbracket(args.IP))
                    print(json.dumps(res, sort_keys=False, indent=4))
                elif args.subcommand == "url":
                    res = vt.get_url_report(args.URL)
                    print(json.dumps(res, sort_keys=False, indent=4))
                elif args.subcommand == "domainlist":
                    print("Not implemented yet with public access, please propose PR if you need it")
                elif args.subcommand == "dl":
                    print("VirusTotal does not allow downloading files with a public feed, sorry")
                    sys.exit(0)
                else:
                    self.parser.print_help()
        else:
            self.parser.print_help()



import tweepy

class Bird(object):
    def __init__(self, config):
        self.consumer_key = config['consumer_key']
        self.consumer_secret = config['consumer_secret']
        self.access_token = config['access_token']
        self.access_token_secret = config['access_token_secret']
        self.api = None

    def _authenticate(self):
        """
        Authenticate on twitter
        """
        auth = tweepy.OAuthHandler(self.consumer_key, self.consumer_secret)
        auth.set_access_token(self.access_token, self.access_token_secret)
        self.api = tweepy.API(auth)

    def get_profile_information(self, username):
        """
        Get profile information on an account
        """
        if self.api is None:
            self._authenticate()

        return self.api.get_user(screen_name=username)

    def get_user_tweets(self, username, since_id=None):
        """
        Download all tweets for an user
        Max is around 3200 tweets
        """
        if self.api is None:
            self._authenticate()
        tweets = []
        if since_id:
            cursor = tweepy.Cursor(self.api.user_timeline, screen_name=username, since_id=since_id)
        else:
            cursor = tweepy.Cursor(self.api.user_timeline, screen_name=username)

        for item in cursor.items():
            tweets.append(item)

        return tweets

    def get_searched_tweets(self, hashtag, since_id=None):
        """
        Search all tweets for a hashtag
        """
        if self.api is None:
            self._authenticate()

        tweets = []
        if since_id:
            cursor = tweepy.Cursor(self.api.search, q=hashtag, count=100, since_id=since_id)
        else:
            cursor = tweepy.Cursor(self.api.search, q=hashtag, count=100)
        try:
            for item in cursor.items():
                tweets.append(item)
        except tweepy.error.TweepError:
            print("Reached Twitter rate limit")
        return tweets

    def get_tweet(self, tweet_id):
        """
        Return a Tweepy status for this id
        """
        if self.api is None:
            self._authenticate()
        return self.api.get_status(tweet_id)



#! /usr/bin/env python
import sys
from dns import resolver, reversename, exception
from harpoon.commands.base import Command
from harpoon.lib.utils import is_ip, unbracket


class CommandDns(Command):
    """
    # DNS plugin

    **Map DNS information for a domain or an IP address**

    * Check DNS information on a domain: `harpoon dns DOMAIN`
    * Check for all types of DNS entries: `harpoon dns -e DOMAIN`

    """
    # TODO: graph output
    # TODO: option to define DNS server
    name = "dns"
    description = "Map DNS information for a domain or an IP"
    config = {}
    all_types = ['NONE', 'A', 'NS', 'MD', 'MF', 'CNAME', 'SOA', 'MB', 'MG',
        'MR', 'NULL', 'WKS', 'PTR', 'HINFO', 'MINFO', 'MX', 'TXT', 'RP',
        'AFSDB', 'X25', 'ISDN', 'RT', 'NSAP', 'NSAP-PTR', 'SIG', 'KEY',
        'PX', 'GPOS', 'AAAA', 'LOC', 'NXT', 'SRV', 'NAPTR', 'KX', 'CERT',
        'A6', 'DNAME', 'OPT', 'APL', 'DS', 'SSHFP', 'IPSECKEY', 'RRSIG',
        'NSEC', 'DNSKEY', 'DHCID', 'NSEC3', 'NSEC3PARAM', 'TLSA', 'HIP',
        'CDS', 'CDNSKEY', 'CSYNC', 'SPF', 'UNSPEC', 'EUI48', 'EUI64',
        'TKEY', 'TSIG', 'IXFR', 'AXFR', 'MAILB', 'MAILA', 'ANY', 'URI',
        'CAA', 'TA', 'DLV']

    def add_arguments(self, parser):
        parser.add_argument('TARGET', help='Domain or IP to query')
        parser.add_argument('--extended', '-e', action='store_true',
                help="Extended testing of all DNS types")
        self.parser = parser

    def owner_to_email(self, owner):
        rev = owner[:-1].split('.')
        return '.'.join(rev[:-2]) + "@" + ".".join(rev[-2:])

    def run(self, conf, args, plugins):
        if is_ip(unbracket(args.TARGET)):
            # That's an IP address
            ptr_n = str(reversename.from_address(unbracket(args.TARGET)))
            try:
                answer = [entry for entry in resolver.query(ptr_n, "PTR")][0]
                print("%s - %s" % (ptr_n, str(answer)))
            except (resolver.NXDOMAIN, resolver.NoAnswer):
                print("%s - %s" % (ptr_n, "NXDOMAIN"))
        else:
            cip = plugins['ip']
            if args.extended:
                for a in self.all_types:
                    try:
                        answers = resolver.query(unbracket(args.TARGET), a)
                        for rdata in answers:
                            print(a, ':', rdata.to_text())
                    except Exception as e:
                        pass
            else:
                target = unbracket(args.TARGET)
                # A
                print("# A")
                try:
                    answers = resolver.query(target, 'A')
                except (resolver.NoAnswer, resolver.NXDOMAIN):
                    print("No A entry")
                except resolver.NoNameservers:
                    print("No A entry: SERVFAIL")
                except exception.Timeout:
                    print("Timeout")
                else:
                    for rdata in answers:
                        info = cip.ipinfo(rdata.address)
                        print("%s: ASN%i %s - %s %s" % (
                                rdata.address,
                                info['asn'],
                                info['asn_name'],
                                info['city'],
                                info['country']
                            )
                        )

                # AA
                print("")
                print("# AAAA")
                try:
                    answers = resolver.query(target, 'AAAA')
                    for rdata in answers:
                        print(rdata.address)
                except (resolver.NoAnswer, resolver.NXDOMAIN):
                    print("No AAAA entry configured")
                except resolver.NoNameservers:
                    print("No AAAA entry: SERVFAIL")
                except exception.Timeout:
                    print("Timeout")

                # DNS Servers
                print("\n# NS")
                try:
                    answers = resolver.query(target, 'NS')
                except (resolver.NoAnswer, resolver.NXDOMAIN, resolver.NoNameservers):
                    # That's pretty unlikely
                    print("No NS entry configured")
                except exception.Timeout:
                    print("Timeout")
                else:
                    for entry in answers:
                        ttarget = str(entry.target)
                        if is_ip(ttarget):
                            # Pretty unlikely
                            info = cip.ipinfo(ttarget)
                            print("%s - ASN%i %s - %s %s" % (
                                    ttarget,
                                    info['asn'],
                                    info['asn_name'],
                                    info['city'],
                                    info['country']
                                )
                            )
                        else:
                            try:
                                ip = [b.address for b in resolver.query(ttarget, 'A')][0]
                            except resolver.NXDOMAIN:
                                # Hostname without IPv4
                                print(ttarget)
                            else:
                                # Hostname
                                info = cip.ipinfo(ip)
                                print("%s - %s - ASN%i %s - %s %s" % (
                                        ttarget,
                                        ip,
                                        info['asn'],
                                        info['asn_name'],
                                        info['city'],
                                        info['country']
                                    )
                                )

                # MX
                print("\n# MX:")
                try:
                    answers = resolver.query(target, 'MX')
                except (resolver.NoAnswer, resolver.NXDOMAIN):
                    print("No MX entry configured")
                except resolver.NoNameservers:
                    print("No MX entry: SERVFAIL")
                except exception.Timeout:
                    print("Timeout")
                else:
                    for rdata in answers:
                        if is_ip(rdata.exchange):
                            # IP directly
                            info = cip.ipinfo(rdata.exchange)
                            print("%i %s - ASN%i %s - %s %s" % (
                                    rdata.preference,
                                    rdata.exchange,
                                    info['asn'],
                                    info['asn_name'],
                                    info['city'],
                                    info['country']
                                )
                            )
                        else:
                            try:
                                ip = [b.address for b in resolver.query(rdata.exchange, 'A')][0]
                            except (resolver.NoAnswer, resolver.NXDOMAIN):
                                # Hostname without IPv4
                                print("%i %s" % (rdata.preference, rdata.exchange))
                            else:
                                # Hostname
                                info = cip.ipinfo(ip)
                                print("%i %s - %s - ASN%i %s - %s %s" % (
                                        rdata.preference,
                                        rdata.exchange,
                                        ip,
                                        info['asn'],
                                        info['asn_name'],
                                        info['city'],
                                        info['country']
                                    )
                                )

                # SOA
                print("\n# SOA")
                try:
                    answers = resolver.query(target, 'SOA')
                except (resolver.NoAnswer, resolver.NXDOMAIN):
                    print("No SOA entry configured")
                except resolver.NoNameservers:
                    print("No SOA entry: SERVFAIL")
                except exception.Timeout:
                    print("Timeout")
                else:
                    entry = [b for b in answers][0]
                    print("NS: %s" % str(entry.mname))
                    print("Owner: %s" % self.owner_to_email(str(entry.rname)))

                # TXT
                print("\n# TXT:")
                try:
                    answers = resolver.query(target, 'TXT')
                except (resolver.NoAnswer, resolver.NXDOMAIN):
                    print("No TXT entry configured")
                except resolver.NoNameservers:
                    print("No TXT entry: SERVFAIL")
                except exception.Timeout:
                    print("Timeout")
                else:
                    for a in answers:
                        print(a.to_text())



#! /usr/bin/env python
import sys
from harpoon.commands.base import Command
from selenium import webdriver

class CommandScreenshot(Command):
    """
    # Screenshot

    **Takes a screenshot of a webpage**

    `harpoon screenshot http://google.com`
    """
    name = "screenshot"
    description = "Takes a screenshot of a webpage"

    def add_arguments(self, parser):
        parser.add_argument('URL', help='URL of the webpage')
        parser.add_argument('--output', '-o', default="screenshot.png", help='Name of the screenshot image saved')
        self.parser = parser

    def run(self, conf, args, plugins):
        try:
            driver = webdriver.PhantomJS()
            driver.set_window_size(1024, 768) # set the window size that you need
            driver.get(args.URL)
            driver.save_screenshot(args.output)
            print('Webpage %s saved in %s' % (args.URL, args.output))
        except WebDriverException:
            print('Install phantomjs to use this module (npm install -g phantomjs)')
            sys.exit(1)



#! /usr/bin/env python
import sys
import os
import json
import time
from harpoon.commands.base import Command
from harpoon.lib.utils import unbracket
from harpoon.lib.ipinfo import IPInfo, IPInfoError


class CommandIPInfo(Command):
    """
    # ipinfo.io plugin

    **Query ipinfo.io API**

    * Get info on an IP : `harpoon ipinfo ip IP`
    * Get infos on a list of IPs in a file : `harpoon ipinfo file FILE`
    """
    name = "ipinfo"
    description = "Request ipinfo.io information"
    config = {'IPInfo': ['token']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('ip', help='Information on an IP')
        parser_a.add_argument('IP', help='IP address')
        parser_a.set_defaults(subcommand='ip')
        parser_b = subparsers.add_parser('file', help='Information on a list of IPs')
        parser_b.add_argument('FILE', help='Filename')
        parser_b.add_argument('--delay', '-d', type=int, default=1, help='Delay between two queries in seconds')
        parser_b.set_defaults(subcommand='file')
        self.parser = parser

    def run(self, conf, args, plugins):
        ipinfo = IPInfo(token=conf['IPInfo']['token'])
        if 'subcommand' in args:
            if args.subcommand == 'ip':
                try:
                    infos = ipinfo.get_infos(unbracket(args.IP))
                except IPInfoError:
                    print("Invalid request")
                else:
                    print(json.dumps(infos,  sort_keys=True, indent=4, separators=(',', ': ')))
            elif args.subcommand == 'file':
                if os.path.isfile(args.FILE):
                    with open(args.FILE) as f:
                        data = f.read().split("\n")
                    print("IP;Hostname;City;Region;Country;Location;Company Name;Company Domain;Company Type;ASN;AS Name;AS Domain;AS Route;AS Type")
                    for d in data:
                        if d.strip() == '':
                            continue
                        ip = unbracket(d.strip())
                        try:
                            infos = ipinfo.get_infos(ip)
                        except IPInfoError:
                            print("%s;;;;;;;;;;;;;" % ip)
                        else:
                            if "company" in infos and "asn" in infos:
                                print("%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s" % (
                                        ip,
                                        infos['hostname'] if 'hostname' in infos else '',
                                        infos['city'] if 'city' in infos else '',
                                        infos['region'] if 'region' in infos else '',
                                        infos['country'] if 'country' in infos else '',
                                        infos['loc'] if 'loc' in infos else '',
                                        infos['company']['name'] if 'company' in infos else '',
                                        infos['company']['domain'] if 'company' in infos else '',
                                        infos['company']['type'] if 'company' in infos else '',
                                        infos['asn']['asn'] if 'asn' in infos['asn'] else '',
                                        infos['asn']['name'] if 'name' in infos['asn'] else '',
                                        infos['asn']['domain'] if 'domain' in infos['asn'] else '',
                                        infos['asn']['route'] if 'route' in infos['asn'] else '',
                                        infos['asn']['type'] if 'type' in infos['asn'] else ''
                                    )
                                )
                            elif "company" in infos and "asn" not in infos:
                                print("%s;%s;%s;%s;%s;%s;%s;%s;%s;;;;;" % (
                                        ip,
                                        infos['hostname'] if 'hostname' in infos else '',
                                        infos['city'],
                                        infos['region'],
                                        infos['country'],
                                        infos['loc'],
                                        infos['company']['name'],
                                        infos['company']['domain'],
                                        infos['company']['type']
                                    )
                                )
                            elif "asn" in infos and "company" not in infos:
                                print("%s;%s;%s;%s;%s;%s;;;;%s;%s;%s;%s;%s" % (
                                        ip,
                                        infos['hostname'] if 'hostname' in infos else '',
                                        infos['city'],
                                        infos['region'],
                                        infos['country'],
                                        infos['loc'],
                                        infos['asn']['asn'],
                                        infos['asn']['name'],
                                        infos['asn']['domain'],
                                        infos['asn']['route'],
                                        infos['asn']['type']
                                    )
                                )
                            else:
                                print("%s;%s;%s;%s;%s;%s;;;;%s;%s;;;" % (
                                        ip,
                                        infos['hostname'] if 'hostname' in infos else '',
                                        infos['city'],
                                        infos['region'],
                                        infos['country'],
                                        infos['loc'],
                                        infos['org'].split(' ')[0],
                                        ' '.join(infos['org'].split(" ")[1:])
                                    )
                                )
                        time.sleep(args.delay)

                else:
                    print("This file does not exist")
                    sys.exit(1)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import os
import io
import consolemd
from harpoon.commands.base import Command


class CommandHelp(Command):
    """
    # Help Command

    **Show help for a command**
    Example : `harpoon help config`
    """
    name = "help"
    description = "Give help on an Harpoon command"

    def add_arguments(self, parser):
        parser.add_argument('COMMAND', help='Show the help of the given command', nargs='?')

    def run(self, conf, args, plugins, parser):
        if args.COMMAND in plugins:
            renderer = consolemd.Renderer()
            # Remove empty space at the beginning of lines
            content = [item.strip() for item in plugins[args.COMMAND].__doc__.splitlines()]
            renderer.render('\n'.join(content))
        else:
            parser.print_help()



#! /usr/bin/env python
import sys
import requests
from harpoon.commands.base import Command
from harpoon.lib.google import Google
from harpoon.lib.yandex import Yandex
from harpoon.lib.archiveis import ArchiveIs
from harpoon.lib.archiveorg import ArchiveOrg
from harpoon.lib.bing import Bing
from harpoon.lib.utils import unbracket


class CommandCache(Command):
    """
    # Cache

    **Check if a page was cached by archive websites, and download cached page**
    Supported platforms for now: Yandex, Google, Archive.is, Archive.org, Bing


    * Check if a page was archived: `harpoon cache https://www.oreilly.com/'`
    * Download a cached page: `harpoon cache -s google -D https://www.oreilly.com/`
    """
    name = "cache"
    description = "Requests webpage cache from different sources"

    def add_arguments(self, parser):
        parser.add_argument('URL', help='URL of the cache')
        parser.add_argument(
            '--source', '-s',
            choices=['all', 'google', 'yandex', 'webarchive', 'archiveis', 'bing'],
            default='all',
            help='Source of the cache'
        )
        parser.add_argument('--dump', '-D', action='store_true', help='Dump data')
        self.parser = parser

    def run(self, conf, args, plugins):
        url = unbracket(args.URL)
        if args.source == 'all':
            if args.dump:
                print("Please specify the source to dump the data")
                sys.exit(1)
            # Google
            google = Google.cache(url)
            if google['success']:
                if 'date' in google:
                    print('Google: FOUND %s (%s)' % (
                        google['cacheurl'],
                        google['date']
                    ))
                else:
                    print('Google: FOUND %s' % (google['cacheurl']))
            else:
                print("Google: NOT FOUND")
            # Yandex
            yandex = Yandex.cache(url)
            if yandex['success']:
                print('Yandex: FOUND %s' % yandex['cacheurl'])
            else:
                print("Yandex: NOT FOUND")
            # Archive.is
            try:
                arch = ArchiveIs.snapshots(url)
                if len(arch) > 0:
                    print('Archive.is: FOUND')
                    for s in arch:
                        print('-%s: %s' % (s['date'], s['archive']))
                else:
                    print('Archive.is: NOT FOUND')
            except requests.exceptions.ConnectTimeout:
                print('Archive.is: TIME OUT')
            # Web Archive
            web = ArchiveOrg.snapshots(url)
            if len(web) > 0:
                print('Archive.org: FOUND')
                for s in web:
                    print('-%s: %s' % (s['date'], s['archive']))
            else:
                print('Archive.org: NOT FOUND')
            # Bing
            bing = Bing.cache(url)
            if bing['success']:
                print('Bing: FOUND %s (%s)' % (
                    bing['cacheurl'],
                    bing['date']
                ))
            else:
                print("Bing: NOT FOUND")

        elif args.source == "google":
            data = Google.cache(url)
            if data['success']:
                if args.dump:
                    print(data['data'])
                else:
                    print('Cache found: %s (%s)' % (
                            data['cacheurl'],
                            data['date']
                        )
                    )
            else:
                print('No Google cache for this url')
        elif args.source == "yandex":
            data = Yandex.cache(url)
            if data['success']:
                if args.dump:
                    print(data['data'])
                else:
                    print('Cache found: %s' % data['cacheurl'])
            else:
                print('Cache not found')
        elif args.source == "bing":
            data = Bing.cache(url)
            if data['success']:
                if args.dump:
                    print(data['data'])
                else:
                    print('Cache found: %s' % data['cacheurl'])
            else:
                print('Cache not found')
        elif args.source == 'archiveis':
            data = ArchiveIs.snapshots(url)
            if len(data) == 0:
                print('No snapshot found')
            else:
                if args.dump:
                    last = sorted(data, key=lambda x: x['date'], reverse=True)[0]
                    res = ArchiveIs.download_cache(last['archive'])
                    print(res['data'])
                else:
                    print('Snapshot founds:')
                    for s in data:
                        print('-%s: %s' % (s['date'], s['archive']))
        elif args.source == "webarchive":
            data = ArchiveOrg.snapshots(url)
            if len(data) > 0:
                if args.dump:
                    last = sorted(data, key=lambda x: x['date'], reverse=True)[0]
                    cache = ArchiveOrg.download_cache(data['archive'])
                    print(cache['data'])
                else:
                    print('Snapshots found:')
                    for s in data:
                        print('-%s: %s' % (s['date'], s['archive']))
            else:
                print("No snapshot found")
        else:
            self.parser.print_help()



import requests
import hmac
import hashlib
import xml.etree.ElementTree as ET
from dateutil.parser import parse


class TotalHashError(Exception):
    pass


class TotalHashNotFound(TotalHashError):
    pass


class TotalHash(object):
    """
    Total hash APi Wrapper
    Check https://totalhash.cymru.com/api-documentation/#code_examples
    """
    def __init__(self, user, key):
        self.user = user
        self.key = key
        self.base_url = "https://api.totalhash.com"

    def _query(self, query):
        return requests.get(self.base_url + query)

    def search(self, query):
        """
        Search for a search term (max 10 results)
        """
        # TODO: check that the query is valid
        sign = hmac.new(
            self.key.encode("utf-8"),
            query.encode("utf-8"),
            hashlib.sha256
        ).hexdigest()
        res = self._query("/search/%s&id=%s&sign=%s" % (query, self.user, sign))
        if res.status_code != 200:
            raise TotalHashError()
        else:
            root = ET.fromstring(res.text)
            result = root.find('result')
            res = {
                'total': int(result.attrib['numFound']),
                'results': [a.text for a in root.findall('.//result/doc/')]
            }
            return res

    def search_all(self, query):
        """
        Return all the results for a query
        """
        sign = hmac.new(
            self.key.encode("utf-8"),
            query.encode("utf-8"),
            hashlib.sha256
        ).hexdigest()
        start = 0
        end = False
        results = {}
        while not end:
            res = self._query("/search/%s&id=%s&sign=%s&start=%i" % (query, self.user, sign, start))
            if res.status_code != 200:
                raise TotalHashError()
            else:
                root = ET.fromstring(res.text)
                if start == 0:
                    result = root.find('result')
                    results = {
                        'total': int(result.attrib['numFound']),
                        'results': [a.text for a in root.findall('.//result/doc/')]
                    }
                    start += len(results['results'])
                    if start >= results['total']:
                        return results
                else:
                    docs = root.findall('.//result/doc/')
                    for a in docs:
                        results['results'].append(a.text)
                    start += len(docs)
                    if start >= results['total']:
                        return results

    def analysis(self, sha):
        """
        Return the analysis of a hash
        """
        sign = hmac.new(
            self.key.encode("utf-8"),
            sha.encode("utf-8"),
            hashlib.sha256
        ).hexdigest()
        res = self._query("/analysis/%s&id=%s&sign=%s" % (sha, self.user, sign))
        if res.status_code != 200:
            if res.status_code == 404:
                raise TotalHashNotFound()
            else:
                raise TotalHashError()
        else:
            root = ET.fromstring(res.text)
            # May not be complete
            results = {
                'sha1': root.attrib['sha1'],
                'md5': root.attrib['md5'],
                'time': parse(root.attrib['time']),
                'url': 'https://totalhash.cymru.com/analysis/?%s' % root.attrib['sha1'],
                'sections': [],
                'detections': [],
                'running_processes': [],
                'calltree': [],
                'network': {'dns': [], 'flows': []},
                'imports': [],
                'processes': []
            }
            try:
                results['magic'] = root.findall('static/magic')[0].attrib['value']
            except IndexError:
                pass
            try:
                results['timestamp'] = parse(root.findall('static/timestamp')[0].attrib['value'])
            except IndexError:
                pass
            try:
                results['pehash'] = root.findall('static/pehash')[0].attrib['value']
            except IndexError:
                pass
            for s in root.findall('static/section'):
                results['sections'].append(s.attrib)
            for i in root.findall('static/imports'):
                results['imports'].append(i.attrib['dll'])
            for a in root.findall('static/av'):
                results['detections'].append(a.attrib)
            for a in root.findall('network-pcap/dns'):
                results['network']['dns'].append(a.attrib)
            for a in root.findall('network-pcap/flows'):
                results['network']['flows'].append(a.attrib)
            for a in root.findall('running_processes/running_process'):
                results['running_processes'].append(a.attrib)
            for a in root.findall('calltree/process_call'):
                results['calltree'].append(a.attrib)

            # Processes
            for p in root.findall('processes/process'):
                pinfo = {
                        'dll': [],
                        'file': [],
                        'process': [],
                        'registry': [],
                        'mutex': []
                }
                for a in p.attrib:
                    pinfo[a] = p.attrib[a]

                for dll in p.findall('dll_handling_section/load_dll'):
                    pinfo['dll'].append(dll.attrib)
                for f in p.findall('filesystem_section/create_file'):
                    pinfo['file'].append(f.attrib)
                for pp in p.findall('process_section/create_process'):
                    pinfo['process'].append(pp.attrib)

                for r in p.findall('registry_section/set_value'):
                    pinfo['registry'].append(r.attrib)
                for m in p.findall('mutex_section/create_mutex'):
                    pinfo['mutex'].append(m.attrib)
                results['processes'].append(pinfo)

            return results

    def usage(self):
        """
        Retrieve API usage limit and count
        """
        sign = hmac.new(
            self.key.encode("utf-8"),
            b'usage',
            hashlib.sha256
        ).hexdigest()
        res = self._query("/usage/id=%s&sign=%s" % (self.user, sign))
        if res.status_code != 200:
            raise TotalHashError()
        else:
            return res.text







#! /usr/bin/env python
import json
from harpoon.commands.base import Command
from harpoon.lib.hibp import HIBP, HibpNotFound


class CommandHibp(Command):
    """
    # Have I Been Pwned

    **Requests Have I Been Pwned API https://haveibeenpwned.com/**

    * Search for an email: `harpoon hibp admin@example.org`
    * Search for leaks in pasts: `harpoon bibp -p admin@example.org`
    * See raw JSON output: `harpoon hibp -j admin@example.org`
    """
    name = "hibp"
    description = "Request Have I Been Pwned API (https://haveibeenpwned.com/)"
    config = {'hibp': ['key']}

    def add_arguments(self, parser):
        parser.add_argument('EMAIL',  help='Email to check')
        parser.add_argument('--pastes', '-p', help='Check pastes instead of breaches', action='store_true')
        parser.add_argument('--json', '-j', action='store_true', help='Check pastes instead of breaches')
        self.parser = parser

    def run(self, conf, args, plugins):
        h = HIBP(conf['hibp']['key'])
        if args.pastes:
            try:
                res = h.get_pastes(args.EMAIL)
            except HibpNotFound:
                print("Account not found in the HIBP database")
            else:
                if args.json:
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    print("Account found in %i pastes:" % len(res))
                    for r in res:
                        print('[+] %s: %s %s "%s"' % (
                                r['Date'],
                                r['Source'],
                                r['Id'],
                                r['Title']
                            )
                        )
        else:
            try:
                res = h.get_breaches_account(args.EMAIL)
            except HibpNotFound:
                print("Account not found in the HIBP database")
            else:
                if args.json:
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    print("Account found in %i breaches\n" % len(res))
                    for r in res:
                        print("%s\n%s\n%s\n" % (
                                r['Name'],
                                r['BreachDate'],
                                r['Description']
                            )
                        )



#! /usr/bin/env python
import sys
from harpoon.commands.base import Command
from harpoon.lib.bitly import Bitly, Link

class CommandBitly(Command):
    """
    # Bit.ly plugin

    **Query bit.ly API**

    * Check a has information: `harpoon bitly -H 1234`
    * Check multiple hashes from a file: `harpoon bitly -f hash.csv`
    """
    name = "bitly"
    description = "Request bit.ly information through the API"
    config = { 'Bitly': ['token']}

    def add_arguments(self, parser):
        parser.add_argument('--hash', '-H', help='HASH of a link')
        parser.add_argument('--file', '-f', help='File containing list of hashes')
        self.parser = parser

    def run(self, conf, args, plugins):
        bitly = Bitly(access_token=conf['Bitly']["token"])
        if args.hash:
            link = Link(bitly, args.hash)
            link.pprint()
        elif args.file:
            f = open(args.file, 'r')
            data = f.read().split()
            print("Date;Short URL;Long URL;Analytics;Aggregate;Aggregate Hash;User;Short URL Clicks;Long URL Clicks")
            for d in data:
                if d.strip() != "":
                    link = Link(bitly, d)
                    print("%s;%s;%s;%s;%s;%s;%s;%i;%i" % (
                            link.timestamp.strftime("%m/%d/%Y %H:%M:%S"),
                            link.short_url,
                            link.long_url,
                            link.short_url + "+",
                            link.hash if link.is_aggregate else link.aggregate.hash,
                            "Yes" if link.is_aggregate else "No",
                            link.user_hash,
                            link.clicks,
                            link.clicks if link.is_aggregate else link.aggregate.clicks
                        )
                    )
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import json
import requests
from harpoon.commands.base import Command


class CommandQuad9(Command):
    """
    # Quad9

    Check if a domain is blocked by Quad9 https://quad9.net/

    `harpoon quad9 DOMAIN`
    """
    name = "quad9"
    description = "Check if a domain is blocked by Quad9"

    def add_arguments(self, parser):
        parser.add_argument('DOMAIN',  help='Domain to be checked')
        parser.add_argument('--type', '-t', default='A',  help='DNS Type')
        parser.add_argument('--verbose', '-v', action='store_true',  help='Display results')
        self.parser = parser

    def run(self, conf, args, plugins):
        params = {
            'name': args.DOMAIN,
            'type': args.type,
            'ct': 'application/dns-json',
        }
        r = requests.get("https://dns.quad9.net:5053/dns-query", params=params)
        if r.status_code != 200:
            print('Problem querying quad9 :(')
            sys.exit(1)
        if r.json()['Status'] == 3:
            print("{} - BLOCKED".format(args.DOMAIN))
        else:
            print("{} - NOT BLOCKED".format(args.DOMAIN))
        if args.verbose:
            print(json.dumps(r.json(), indent=4))



#! /usr/bin/env python
import json
from harpoon.commands.base import Command
from harpoon.lib.opencage import OpenCageGeocode


class CommandOpenCageInfo(Command):
    """
    # OpenCage plugin

    **Query OpenCage geocoder API**

    * Forward Geocoding : `harpoon opencage search QUERY`
    * Reverse Geocoding : `harpoon opencage reverse LATITUDE LONGITUDE`
    """
    name = "opencage"
    description = "Forward/Reverse Geocoding using OpenCage"
    config = {'OpenCage': ['key']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('search', help='Search coordinates of an address or location')
        parser_a.add_argument('QUERY', help='Query (Address, Location, ...)')
        parser_a.set_defaults(subcommand='search')
        parser_b = subparsers.add_parser('reverse', help='Request the nearest address for coordinates')
        parser_b.add_argument('LATITUDE', help='Latitude')
        parser_b.add_argument('LONGITUDE', help='Longitude')
        parser_b.set_defaults(subcommand='reverse')
        self.parser = parser

    def run(self, conf, args, plugins):
        geocoder = OpenCageGeocode(key=conf['OpenCage']['key'])
        if 'subcommand' in args:
            if args.subcommand == 'search':
                try:
                    infos = geocoder.geocode(args.QUERY)
                except OpenCageError:
                    print("Invalid request")
                else:
                    print(json.dumps(infos,  sort_keys=True, indent=4, separators=(',', ': ')))
            elif args.subcommand == 'reverse':
                try:
                    infos = geocoder.reverse_geocode(args.LATITUDE, args.LONGITUDE)
                except OpenCageError:
                    print("Invalid request")
                else:
                    print(json.dumps(infos,  sort_keys=True, indent=4, separators=(',', ': ')))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import os
import sys
import json
import requests
import pyasn
import gzip
import urllib.request
from harpoon.commands.base import Command
from harpoon.lib.utils import bracket, unbracket

class CommandAsn(Command):
    """
    # ASN command

    **Get ASN information from different sources**

    * Offline ASN info: `harpoon asn info ASN15134`
    * ASN Info from peering db (https://peeringdb.com/) : `harpoon asn db 1228` (-j for raw output)
    * List subnets for an ASN: `harpoon asn subnet 15348`

    Example:
    ```
    $ harpoon asn db 18182
    Name: So-net Entertainment Taiwan Limited
    Website: http://www.so-net.net.tw
    Created: 2011-06-15T06:16:44Z
    ```
    """
    name = "asn"
    description = "Gather information on an ASN"
    config = None
    update_needed = True
    asn_name = os.path.join(os.path.expanduser('~'), '.config/harpoon/asnnames.csv')
    asncidr = os.path.join(os.path.expanduser('~'), '.config/harpoon/asncidr.dat')
    asncaida = os.path.join(os.path.realpath(__file__)[:-16], 'data/caida.txt')

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('info', help='Information on an ASn number')
        parser_a.add_argument('ASN', help='ASN Number')
        parser_a.set_defaults(subcommand='info')
        parser_b = subparsers.add_parser('db', help='ASN information from peeringdb (https://peeringdb.com/)')
        parser_b.add_argument('ASN', help='ASN Number')
        parser_b.add_argument('--json', '-j', help='Show raw json', action='store_true')
        parser_b.set_defaults(subcommand='db')
        parser_c = subparsers.add_parser('subnet', help='List of subnets for an ASN number')
        parser_c.add_argument('ASN', help='ASN Number')
        parser_c.set_defaults(subcommand='subnet')
        self.parser = parser

    def asn_caida(self, asn):
        """
        Read the 2015 CAIDA database and returns the classification for the given ASN
        input: asn : integer
        output {'source': SOURCE, 'type': TYPE}
        """
        with open(self.asncaida, 'r') as f:
            line = f.readline()
            while line != '':
                if line.startswith('#'):
                    line = f.readline()
                    continue
                data = line.split("|")
                if int(data[0]) == asn:
                    return {'source': data[1], 'type': data[2].strip()}
                line = f.readline()
        return {'source': '', 'type': 'Unknown'}

    def asnname(self, asn):
        """
        Search for the ASN name based on its number
        Input: asnnumber (integer)
        returns: string
        Returns an empty string if not found
        """
        # Search for name
        f = open(self.asn_name, 'r')
        line = f.readline()
        while line != '':
            s = line.split('|')
            if s[0] == str(asn):
                f.close()
                return s[1].strip()
            line = f.readline()
        f.close()
        return ''

    def run(self, conf, args, plugins):
        if hasattr(args, 'ASN'):
            if args.ASN.lower().startswith("asn"):
                asn = int(args.ASN[3:])
            elif args.ASN.lower().startswith("as"):
                asn = int(args.ASN[2:])
            else:
                asn = int(args.ASN)
        else:
            self.parser.print_help()
            sys.exit(0)
        if 'subcommand' in args:
            if args.subcommand == 'info':
                info = self.asnname(asn)
                if len(info):
                    print("ASN%i - %s" % (asn, info))
                else:
                    print("Unknown ASN")
            elif args.subcommand == "subnet":
                asndb = pyasn.pyasn(self.asncidr)
                subnets = asndb.get_as_prefixes(asn)
                for s in subnets:
                    print(s)
            elif args.subcommand == "db":
                r = requests.get('https://peeringdb.com/api/net?asn=%i' % asn)
                if r.status_code == 200:
                    if args.json:
                        print(json.dumps(r.json(), sort_keys=False, indent=4))
                    else:
                        data = r.json()['data'][0]
                        print('Name: %s' % data['name'])
                        if data['aka'] != '':
                            print("aka: %s" % data['aka'])
                        if data['notes'] != '':
                            print("Notes: %s" % data['notes'])
                        if data['website'] != '':
                            print("Website: %s" % data['website'])
                        print("Created: %s" % data['created'])
                else:
                    print("ASN not found")
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



import requests
from requests.auth import HTTPBasicAuth


class HybridAnalysisFailed(Exception):
    pass


class HybridAnalysis(object):
    def __init__(self, key, secret):
        self.key = key
        self.secret = secret
        self.base_url = "https://www.hybrid-analysis.com/api/"
        # Mandatory for Hybrid Analysis (weird)
        self.ua = "Mozilla/5.0 (Android 4.4; Mobile; rv:41.0) Gecko/41.0 Firefox/41.0"

    def _request(self, path, data={}):
        """
        Query the API with the given path
        """
        headers = { 'User-Agent': self.ua }
        r = requests.get(self.base_url + path, auth=HTTPBasicAuth(self.key, self.secret), headers=headers, params=data)
        if r.status_code != 200:
            raise HybridAnalysisFailed()
        else:
            res = r.json()
            if 'response_code' in res:
                if res['response_code'] == 0:
                    return res['response']
            raise HybridAnalysisFailed()

    def quota(self):
        """
        Query quota information
        """
        return self._request("quota")

    def get_report(self, hash):
        """
        Search for a report for the given hash
        """
        return self._request('scan/' + hash)

    def search(self, query):
        """
        Search for a query in Hybrid Analysis
        """
        return self._request('search', {'query': query})

    def get_summary(self, hash, envid):
        """
        Request the analysis summary
        """
        return self._request('summary/' + hash, data = {'environmentId': envid})

    def get_last_analysis(self, hash):
        """
        Get details from the last analysis
        """
        # Query the hash and then the analysis
        res = self.get_report(hash)
        last = sorted(res, key=lambda x:x['analysis_start_time'], reverse=True)[0]
        return self.get_summary(hash, last['environmentId'])



#! /usr/bin/env python
import sys
import json
import hashlib
from harpoon.commands.base import Command
from harpoon.lib.totalhash import TotalHash, TotalHashNotFound
from harpoon.lib.utils import json_serial


class CommandTotalHash(Command):
    """
    # Total Hash

    **Query Total hash API https://totalhash.cymru.com/**

    * Check API usage: `harpoon totalhash usage`
    * Search for a file `harpoon totalhash search mutex:ASPLOG`
    * Search with all results (/!\ earch 10 results is one query): `harpoon totalhash search -a mutex:ASPLOG`
    * Get a file analysis: `harpoon totalhash hash HASH`
    """
    name = "totalhash"
    description = "Request Total Hash API"
    config = {'TotalHash': ['key', 'user']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('hash', help='Request info on a hash')
        parser_a.add_argument('HASH', help='Hash')
        parser_a.set_defaults(subcommand='hash')
        parser_f = subparsers.add_parser('usage', help='Print information on TotalHash API usage')
        parser_f.set_defaults(subcommand='usage')
        parser_e = subparsers.add_parser('search', help='Search in Total Hash database')
        parser_e.add_argument('QUERY', help='query')
        parser_e.add_argument('--all', '-a', help='Query all results', action='store_true')
        parser_e.set_defaults(subcommand='search')
        self.parser = parser

    def run(self, conf, args, plugins):
        th = TotalHash(conf['TotalHash']['user'], conf['TotalHash']['key'])
        if 'subcommand' in args:
            if args.subcommand == 'usage':
                print(th.usage())
            elif args.subcommand == "search":
                if args.all:
                    res = th.search_all(args.QUERY)
                else:
                    res = th.search(args.QUERY)
                print('%i files found:\n' % res['total'])
                for r in res['results']:
                    print(r)
            elif args.subcommand == 'hash':
                try:
                    res = th.analysis(args.HASH)
                except TotalHashNotFound:
                    print("File not found")
                else:
                    print(json.dumps(res, sort_keys=True, indent=4, separators=(',', ': '), default=json_serial))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



import argparse
import json
import requests
import os
import sys
try:
    import configparser as cp
except ImportError:
    # python2
    import ConfigParser as cp

class GoogleShortener(object):
    def __init__(self, token):
        self.host = 'https://www.googleapis.com/urlshortener/v1/url'
        self.token = token

    def get_analytics(self, hash):
        params = {'key': self.token, 'shortUrl': 'http://goo.gl/' + hash, 'projection': 'FULL'}
        r = requests.get(self.host, params=params)
        return r.json()

    def expand(self, hash):
        params = {'key': self.token, 'shortUrl': 'http://goo.gl/' + hash}
        r = requests.get(self.host, params=params)
        return r.json()

    def shorten(self, long_url):
        params = {'key': self.token, 'longUrl': long_url}
        r = requests.post(self.host, data=params)
        return r.json()



#! /usr/bin/env python
import sys
import json
import tweepy
import time
from harpoon.commands.base import Command
from harpoon.lib.bird import Bird

class CommandTwitter(Command):
    """
    # Twitter

    **Dumps information from Twitter**
    """
    name = "twitter"
    description = "Requests Twitter API"
    config = {'Twitter': ['consumer_secret', 'consumer_key', 'access_token', 'access_token_secret']}

    def add_arguments(self, parser):
        parser.add_argument('--user', '-u',
	    	help='Get user infos')
        parser.add_argument('--tweets', '-t',
            help='Download tweets of an user')
        parser.add_argument('--tweet', '-T',
            help='Download tweet with the given id')
        parser.add_argument('--save', '-s',
            help='save all infos about an user and their tweets')
        parser.add_argument('--file', '-f',
            help='File containing usernames, display user infos in CSV format')
        self.parser = parser

    def run(self, conf, args, plugins):
        bird = Bird(conf['Twitter'])

        if args.user:
            a = bird.get_profile_information(args.user)
            print(json.dumps(a._json, sort_keys=True, indent=4, separators=(',', ': ')))
        elif args.tweets:
            a = bird.get_user_tweets(args.tweets)
            for page in a:
                # FIXME : improve this
                print(json.dumps(page._json, sort_keys=True, indent=4, separators=(',', ': ')))
        elif args.tweet:
            a = bird.get_tweet(args.tweet)
            print(json.dumps(a._json, sort_keys=True, indent=4, separators=(',', ': ')))
        elif args.save:
            data = {}
            a = bird.get_profile_information(args.save)
            data["user"] = a._json
            b = bird.get_user_tweets(args.save)
            data["tweets"] = []
            for t in b:
                data["tweets"].append(t._json)
            print(json.dumps(data))
        elif args.file:
            f = open(args.file, 'r')
            data = f.read().split()
            f.close()
            print("Handle;Status;Name;Id;Description;url;Location;Time zone;UTC offset;Created at;Last Tweet;lang;Tweet count;Favourite count;Followers count;Following count;List count;Verified;Geo enabled;Default profile;Default profile image;Contributors Enabled")
            for d in data:
                try:
                    user = bird.get_profile_information(d.strip())
                    print("%s;Active;%s;%i;%s;%s;%s;%s;%i;%s;%s;%s;%i;%i;%i;%i;%i;%s;%s;%s;%s;%s" %
                        (
                            user.screen_name,
                            user.name,
                            user.id,
                            user.description.replace(";", ",").replace("\n", ""),
                            user.entities['url']['urls'][0]['expanded_url'] if user.url is not None else "",
                            user.location,
                            user.time_zone,
                            user.utc_offset if user.utc_offset is not None else 0,
                            user.created_at.strftime("%m/%d/%Y %H:%M:%S"),
                            user.status.created_at.strftime("%m/%d/%Y %H:%M:%S") if user.status is not None else "",
                            user.lang,
                            user.statuses_count,
                            user.favourites_count,
                            user.followers_count,
                            user.friends_count,
                            user.listed_count,
                            user.verified,
                            user.geo_enabled,
                            user.default_profile,
                            user.default_profile_image,
                            user.contributors_enabled
                        )
                    )
                except tweepy.error.TweepError as ex:
                    if ex.args[0][0]['code'] == 88:
                        # Rate limit exceeded
                        sys.stderr.write("Rate Limit exceeded\n")
                        sys.exit(1)
                    elif ex.args[0][0]['code'] == 50:
                        print("%s;Not Found;;;;;;;;;;;;;;;;;;;;" % d.strip())
                    elif ex.args[0][0]['code'] == 63:
                        print("%s;Suspended;;;;;;;;;;;;;;;;;;;;" % d.strip())
                    else:
                        sys.stderr.write("Weird error %i: %s\n" % (ex.args[0][0]['code'], ex.args[0][0]['message']))
                        sys.exit(1)
                time.sleep(1)
        else:
            print("Please provide a command")
            self.parser.print_help()



import requests
from bs4 import BeautifulSoup
from urllib.parse import urlencode
from dateutil.parser import parse
from harpoon.lib.utils import same_url

class Bing(object):
    @staticmethod
    def search(query):
        """
        Search in bing
        """
        # FIXME : change default UA
        r = requests.get(
            "https://www.bing.com/search",
            params = {'q': query }
        )
        soup = BeautifulSoup(r.text, 'lxml')
        res = []
        divs = soup.find_all('li', class_='b_algo')
        for d in divs:
            data = {
                'name': d.a.text,
                'url': d.a['href'],
                'text': d.p.text
            }
            attribution = d.find('div', class_='b_attribution')
            # Check if cache infos in attribute
            if 'u' in attribution.attrs:
                b = attribution['u'].split('|')
                data['cache'] = "http://cc.bingj.com/cache.aspx?d=%s&w=%s" % (
                    b[2],
                    b[3]
                )
            res.append(data)
        return res

    @staticmethod
    def download_cache(url):
        """
        Download cache data from a cached page
        """
        r = requests.get(url)
        if r.status_code == 200:
            if "Could not find the requested document in the cache" in r.text:
                # Bing bug
                return {"success": False}
            else:
                soup = BeautifulSoup(r.text, 'lxml')
                content = soup.find('div', class_='cacheContent')
                data = r.text[r.text.find('<div class="cacheContent">')+26:len(r.text)-41]
                return {
                    "success": True,
                    "date": parse(soup.find_all('strong')[1].text),
                    "data": str(content)[26:-40],
                    'url': soup.strong.a['href'],
                    'cacheurl': url
                }
        else:
            if r.status_code != 404:
                print('Weird, it should return 200 or 404')
            return {"success": False}

    @staticmethod
    def cache(url):
        """
        Search for an url in Bing cache
        """
        res = Bing.search(url)
        for i in res:
            if same_url(url, i['url']):
                if 'cache' in i:
                    return Bing.download_cache(i['cache'])
        return {'success': False}




#! /usr/bin/env python
import os
import json
import telethon
import csv
import sys
import time
from telethon import TelegramClient
from telethon.errors import SessionPasswordNeededError
from telethon.tl.functions.channels import GetParticipantsRequest
from telethon.tl.types import ChannelParticipantsSearch
from telethon.errors.rpc_error_list import ChatAdminRequiredError
from harpoon.commands.base import Command
from datetime import date, datetime


def json_serial(obj):
    if isinstance(obj, (datetime, date)):
        return obj.isoformat()
    if isinstance(obj, bytes):
        return obj.decode('utf-8', 'ignore')
    raise TypeError("Type %s not serializable" % type(obj))


class CommandTelegram(Command):
    """
    # Telegram

    **Dump information from Telegram**

    * Get an information on an user : harpoon telegram id USER
    * Dump messages of a channel in json : harpoon telegram messages -f json CHANNEL
    * Dump messages of a channel with media : harpoon telegram messages -f json -D media CHANNEL
    * List users in a group : harpoon telegram users CHANNEL
    """
    name = "telegram"
    description = "Request information from Telegram through the API"
    config = {'Telegram': ['id', 'hash', 'phone']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommands')
        parser_a = subparsers.add_parser('id', help='Query information on a phone number or handle')
        parser_a.add_argument('ID', help='Id to be requested')
        parser_a.set_defaults(subcommand='id')
        parser_b = subparsers.add_parser('messages', help='Get messages from a public channel')
        parser_b.add_argument('ID', help='Id to be requested')
        parser_b.add_argument('--limit', '-l', default=25, type=int,
                help="Limit number of messages to get")
        parser_b.add_argument('--format', '-f', default='text', choices=['text', 'csv', 'json'], help="Output format")
        parser_b.add_argument('--dump', '-D', help="Dump media in the given path")
        parser_b.set_defaults(subcommand='messages')
        parser_c = subparsers.add_parser('users', help='Get user list of a group')
        parser_c.add_argument('ID', help='Id to be requested')
        parser_c.add_argument('--limit', '-l', default=25, type=int,
                help="Limit number of messages to get")
        parser_c.add_argument('--format', '-f', default='text', choices=['text', 'csv', 'json'],
                help="Output format")
        parser_c.set_defaults(subcommand='users')
        self.parser = parser

    def run(self, conf, args, plugins):
        session_file = os.path.join(os.path.expanduser("~"), ".config/harpoon/telegram")
        client = TelegramClient(session_file, int(conf['Telegram']['id']), conf['Telegram']['hash'])
        # FIXME : do not connect if it's help
        client.connect()
        if not client.is_user_authorized():
            client.send_code_request(conf['Telegram']['phone'])
            code_ok = False
            while not code_ok:
                code = input("Enter Telegram code:")
                try:
                    code_ok = client.sign_in(conf['Telegram']['phone'], code)
                except SessionPasswordNeededError:
                    # FIXME: getpass is not imported, that would not work
                    password = getpass('Two step verification enabled. Please enter your password: ')
                    code_ok = client.sign_in(password=password)
        if hasattr(args, 'subcommand'):
            if args.subcommand == 'id':
                try:
                    res = client.get_entity(args.ID)
                    print(json.dumps(res.to_dict(), sort_keys=True, indent=4, default=json_serial))
                except ValueError:
                    print('Identifier not found')
            elif args.subcommand == "messages":
                entity = client.get_entity(args.ID)
                messages = client.get_messages(entity, args.limit)
                users = {}
                if args.dump:
                    if not os.path.exists(args.dump):
                         os.makedirs(args.dump)
                if args.format == "text":
                    if len(messages) == 0:
                        print("No messages in this channel")
                    else:
                        print("%i messages downloaded:" % len(messages))
                        for msg in messages:
                            if isinstance(msg, telethon.tl.types.MessageService):
                                if isinstance(msg.action, telethon.tl.types.MessageActionChatEditPhoto):
                                    print("[%s] Channel Photo Changed" % msg.date.isoformat())
                                elif isinstance(msg.action, telethon.tl.types.MessageActionChannelCreate):
                                    print("[%s] Channel Created" % msg.date.isoformat())
                                elif isinstance(msg.action, telethon.tl.types.MessageActionChatAddUser):
                                    print("[%s] Add Users To the Chat - %s" % (
                                            msg.date.isoformat(),
                                            ", ".join([str(a) for a in msg.action.users])
                                        )
                                    )
                                elif isinstance(msg.action, telethon.tl.types.MessageActionChatDeleteUser):
                                    print("[%s] Remove User from the chat - %i" % (
                                            msg.date.isoformat(),
                                            msg.action.user_id
                                        )
                                    )
                                else:
                                    print("[%s] Message Service: %s" % (
                                            msg.date.isoformat(),
                                            msg.action.message
                                        )
                                    )
                            else:
                                if msg.media is None:
                                    if entity.megagroup:
                                        if msg.from_id not in users:
                                            users[msg.from_id] = client.get_entity(msg.from_id)
                                        print("[%s][%i - @%s] %s" % (
                                                msg.date.isoformat(),
                                                msg.from_id,
                                                users[msg.from_id].username,
                                                msg.message
                                            )
                                        )
                                    else:
                                        print("[%s] %s (%i views)" % (
                                                msg.date.isoformat(),
                                                msg.message,
                                                msg.views
                                            )
                                        )
                                else:
                                    if msg.views:
                                        print("[%s] Media (%i views)" % (
                                                msg.date.isoformat(),
                                                msg.views
                                            )
                                        )
                                    else:
                                        print("[%s] Media" % (msg.date.isoformat()))
                                    if args.dump:
                                        if not os.path.exists(os.path.join(args.dump, str(msg.id) + '.jpg')):
                                            client.download_media(msg.media, os.path.join(args.dump, str(msg.id)))
                                            time.sleep(7)
                elif args.format == "json":
                    msg = [m.to_dict() for m in messages]
                    print(json.dumps(msg, sort_keys=True, indent=4, default=json_serial))
                    if args.dump:
                        for msg in messages:
                            if msg.media is None:
                                if not os.path.exists(os.path.join(args.dump, str(msg.id)+ '.jpg')):
                                    client.download_media(msg.media, os.path.join(args.dump, str(msg.id)))
                                    time.sleep(7)
                elif args.format == "csv":
                    if entity.megagroup:
                        # Chat
                        w = csv.writer(sys.stdout, delimiter=';')
                        w.writerow(["Date", "id", "Username", "userid", "Type", "Message"])
                        for m in messages:
                            if m.from_id not in users:
                                users[m.from_id] = client.get_entity(m.from_id)
                            if isinstance(m, telethon.tl.types.MessageService):
                                w.writerow([m.date.isoformat(), m.id, users[m.from_id].username, m.from_id, m.__class__.__name__, m.action.__class__.__name__])
                            else:
                                w.writerow([m.date.isoformat(), m.id, users[m.from_id].username, m.from_id, m.__class__.__name__, m.message])
                    else:
                        w = csv.writer(sys.stdout, delimiter=';')
                        w.writerow(["Date", "id", "Type", "Information", "Media", "Views"])
                        for m in messages:
                            if isinstance(m, telethon.tl.types.MessageService):
                                if isinstance(m.action, telethon.tl.types.MessageActionChatEditPhoto):
                                    w.writerow([m.date.isoformat(), m.id,  m.__class__.__name__, "Channel Photo Changed", "No", ""])
                                elif isinstance(m.action, telethon.tl.types.MessageActionChannelCreate):
                                    w.writerow([m.date.isoformat(), m.id, m.__class__.__name__, "Channel Created", "No", ""])
                                else:
                                    w.writerow([m.date.isoformat(), m.id, m.__class__.__name__, m.action.__class__.__name__, "No", ""])
                            else:
                                if m.media is None:
                                    # message
                                    w.writerow([m.date.isoformat(), m.id, m.__class__.__name__, m.message, "No", m.views])
                                else:
                                    w.writerow([m.date.isoformat(), m.id, m.__class__.__name__, m.message, "Yes", m.views])
                                    if args.dump:
                                        if not os.path.exists(os.path.join(args.dump, str(m.id) + '.jpg')):
                                            client.download_media(m.media, os.path.join(args.dump, str(m.id)))
                                            # Sleep to avoid being banned
                                            time.sleep(7)
                else:
                    print("Not implemented yet, sorry!")
            elif args.subcommand == "users":
                # List users from a group
                try:
                    entity = client.get_entity(args.ID)
                    offset = 0
                    limit = args.limit
                    all_participants = []

                    while True:
                        participants = client.invoke(GetParticipantsRequest(entity, ChannelParticipantsSearch(''), offset, limit, hash=0))
                        if not participants.users:
                            break
                        all_participants.extend(participants.users)
                        offset += len(participants.users)
                        time.sleep(1)  # This line seems to be optional, no guarantees!
                except ChatAdminRequiredError:
                    print("You don't have required access rights to get this list")
                else:
                    if args.format == "text":
                        for p in all_participants:
                            print("[+] User: %s (%s %s)" % (p.username, p.first_name, p.last_name))
                    elif args.format == "json":
                        users = [u.to_dict() for u in all_participants]
                        print(json.dumps(users, sort_keys=True, indent=4, default=json_serial))
                    else:
                        print("Not implemented yet, sorry!")

            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import pypdns
from harpoon.commands.base import Command
from harpoon.lib.utils import json_serial, unbracket
import json


class CommandCircl(Command):
    """
    # Circl plugin

    **Query CIRCL passive DNS database (https://www.circl.lu/services/passive-dns/)**

    * Search for a domain : `harpoon circl DOMAIN`
    """
    name = "circl"
    description = "Request the CIRCL passive DNS database"
    config = {'Circl': ['user', 'pass']}

    def add_arguments(self, parser):
        parser.add_argument('DOMAIN', help='Domain')
        self.parser = parser

    def run(self, conf, args, plugins):
        x = pypdns.PyPDNS(
                basic_auth=(
                    conf['Circl']['user'],
                    conf['Circl']['pass']
                )
        )
        res = x.query(unbracket(args.DOMAIN))
        print(json.dumps(res, sort_keys=True, indent=4, separators=(',', ': '), default=json_serial))



#! /usr/bin/env python
import sys
import os
import json
import datetime
import urllib.request
import tarfile
import geoip2.database
import re
import subprocess
import glob
import shutil
import pyasn
from IPy import IP
from dateutil.parser import parse
from harpoon.commands.base import Command
from harpoon.lib.utils import bracket, unbracket
from OTXv2 import OTXv2, IndicatorTypes
from passivetotal.libs.whois import WhoisRequest
from github import Github, UnknownObjectException
from harpoon.lib.pgp import Pgp


class CommandEmail(Command):
    """
    # Email

    **Gathers information on an email address**

    """
    name = "email"
    description = "Gather information on an email address"
    config = None

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_b = subparsers.add_parser('intel', help='Gather Threat Intelligence information on an email address')
        parser_b.add_argument('EMAIL', help='Email address')
        parser_b.set_defaults(subcommand='intel')
        self.parser = parser

    def run(self, conf, args, plugins):
        if 'subcommand' in args:
            if args.subcommand == "intel":
                domains = []
                # TODO
                # MISP
                # HIBP
                # Full Contact / Hunter
                # Google / Yandex / Bing
                # Start with MISP and OTX to get Intelligence Reports
                print('###################### %s ###################' % args.EMAIL)
                # OTX
                # Stuck for now because of https://github.com/AlienVault-OTX/OTX-Python-SDK/issues/41
                #otx_e = plugins['otx'].test_config(conf)
                #if otx_e:
                    #print('[+] Downloading OTX information....')
                    #otx = OTXv2(conf["AlienVaultOtx"]["key"])
                    #res = otx.get_indicator_details_full(IndicatorTypes.EMAIL, unbracket(args.EMAIL))

                # PT
                pt_e = plugins['pt'].test_config(conf)
                if pt_e:
                    print('----------------- Passive Total')
                    client = WhoisRequest(conf['PassiveTotal']['username'], conf['PassiveTotal']['key'])
                    raw_results = client.search_whois_by_field(query=args.EMAIL.strip(), field="email")
                    if "results" in raw_results:
                        if len(raw_results["results"]) == 0:
                            print("Nothing found")
                        else:
                            for res in raw_results["results"]:
                                # Concat all other interesting infos
                                other_infos = []
                                for a in ["admin", "tech", "registrant"]:
                                    if a in res:
                                        for b in ["email", "name", "organization", "telephone"]:
                                            if b in res[a]:
                                                if res[a][b] not in other_infos:
                                                    other_infos.append(res[a][b])
                                print("[%s] %-25s (%s)" % (
                                        res["registered"][:10],
                                        res["domain"],
                                        ', '.join(other_infos)
                                    )
                                )
                    else:
                        print("Nothing found")

                    # Github
                    github_e = plugins['github'].test_config(conf)
                    if github_e:
                        print('----------- Github')
                        g = Github(conf['Github']['token'])
                        res = g.search_code(args.EMAIL)
                        nb = 0
                        for i in res:
                            print('[+] %s' % i.html_url)
                            print(i.decoded_content[:300])
                            print('')
                            nb += 1
                            if nb > 10:
                                break
                        if nb == 0:
                            print("Nothing found")

                    # GPG
                    print('----------- PGP server')

                    res = Pgp.search(args.EMAIL)
                    if len(res):
                        for r in res:
                            print("[+] %s\t%s\t%s %s" % (
                                    r['id'],
                                    r['date'].strftime("%Y-%m-%d"),
                                    r['emails'][0][0],
                                    r['emails'][0][1]
                                )
                            )
                            if len(r['emails']) > 1:
                                for e in r['emails'][1:]:
                                    print("\t\t\t\t\t%s %s" % (e[0], e[1]))
                    else:
                        print("Nothing found")




            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



import requests
import re
from bs4 import BeautifulSoup
from dateutil.parser import parse


class Pgp(object):
    @staticmethod
    def search(search):
        url = "http://pgp.mit.edu/pks/lookup?search=" + search
        r = requests.get(url)
        if "No results found" in r.text:
            # Nothing found
            return []
        res = []
        soup = BeautifulSoup(r.text, 'lxml')
        emailsearch = re.compile("([\w\(\) ]+) &lt;([\w@\.]+)&gt;")
        for pre in soup.find_all('pre')[1:]:
            key = {}
            t = str(pre)
            key['revoked'] = "*** KEY REVOKED ***" in t
            key['id'] = pre.a['href'][26:]
            a = t.find('</a>')
            key['date'] = parse(t[a+5:a+15])
            key['emails'] = []
            # get ready for regex
            for i in emailsearch.findall(t):
                key['emails'].append([i[0].strip(), i[1].strip()])

            res.append(key)
        return res



#! /usr/bin/env python
import sys
import json
import censys
from censys import ipv4, certificates, query
from harpoon.commands.base import Command


class CommandCensys(Command):
    """
    # Censys plugin

    **Query Censys.io API**

    * Query information on an IP: `harpoon censys ip 172.217.2.174`
    * Query a certificate: `harpoon censys certificate ID`
    * Search for subdomains based on certificates : `harpoon censys subdomains DOMAIN`
    """
    name = "censys"
    description = "Request information from Censys database (https://censys.io/)"
    config = {'Censys': ['id', 'secret']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('ip', help='Get information on an IP address')
        parser_a.add_argument('IP', help='IP to be searched')
        parser_a.add_argument('--search', '-s', action='store_true',
                help='Search for this value in IP infos')
        parser_a.set_defaults(subcommand='ip')
        parser_b = subparsers.add_parser('cert', help='Get information on a certificate')
        parser_b.add_argument('ID', help='ID of the certificate')
        parser_b.set_defaults(subcommand='cert')
        parser_c = subparsers.add_parser('subdomains', help='Query certificates for a domain looking for subdomains')
        parser_c.add_argument('DOMAIN', help='Domain')
        parser_c.add_argument('--verbose', '-v', action='store_true',
                help='Verbose')
        parser_c.set_defaults(subcommand='subdomains')
        self.parser = parser

    def get_subdomains(self, conf, domain, verbose, only_sub=False):
        """
        Inspired from https://github.com/appsecco/the-art-of-subdomain-enumeration/blob/master/censys_subdomain_enum.py
        """
        api = certificates.CensysCertificates(conf['Censys']['id'], conf['Censys']['secret'])
        subdomains = set()
        for cert in api.search(domain):
            if cert['parsed.subject_dn'].endswith(domain):
                if verbose:
                    print('Certificate : %s - %s' % (
                        cert['parsed.subject_dn'],
                        cert['parsed.fingerprint_sha256']
                        )
                    )
                subdomains.add(cert['parsed.subject_dn'].split('CN=')[1])
                c = api.view(cert['parsed.fingerprint_sha256'])
                try:
                    for name in c['parsed']['names']:
                        if only_sub:
                            if name.endswith(domain):
                                subdomains.add(name)
                        else:
                            subdomains.add(name)
                except KeyError:
                    pass

        return subdomains

    def run(self, conf, args, plugins):
        if 'subcommand' in args:
            if args.subcommand == 'ip':
                api = ipv4.CensysIPv4(conf['Censys']['id'], conf['Censys']['secret'])
                if args.search:
                    res = api.search(args.IP)
                    for r in res:
                        if len(r['ip']) > 11:
                            print("[+] %s\t[Location: %s] [Ports: %s]" % (
                                    r['ip'],
                                    r['location.country'],
                                    " ".join(r['protocols'])
                                )
                            )
                        else:
                            print("[+] %s\t\t[Location: %s] [Ports: %s]" % (
                                    r['ip'],
                                    r['location.country'],
                                    " ".join(r['protocols'])
                                )
                            )
                else:
                    try:
                        ip = api.view(args.IP)
                        print(json.dumps(ip, sort_keys=True, indent=4, separators=(',', ': ')))
                    except censys.base.CensysNotFoundException:
                        print('IP not found')
            elif args.subcommand == 'cert':
                try:
                    c = certificates.CensysCertificates(conf['Censys']['id'], conf['Censys']['secret'])
                    res = c.view(args.ID)
                except censys.base.CensysNotFoundException:
                    print("Certificate not found")
                else:
                    print(json.dumps(res, sort_keys=True, indent=4, separators=(',', ': ')))
            elif args.subcommand == 'subdomains':
                subdomains = self.get_subdomains(conf, args.DOMAIN, args.verbose)
                for d in subdomains:
                    print(d)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import json
import datetime
from harpoon.commands.base import Command
from harpoon.lib.utils import bracket, unbracket
from dnsdb import Dnsdb


class DnsDbTotal(Command):
    """
    # Farsight DnsDB

    * On a domain : `harpoon dnsdb domain DOMAIN`
    * On an IP : `harpoon dnsdb ip IP`
    """
    name = "dnsdb"
    description = "Requests Farsight DNSDB"
    config = {'Dnsdb': ['key']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('ip', help='Request passive DNS info on an IP address')
        parser_a.add_argument('IP',  help='IP to be queried')
        parser_a.set_defaults(subcommand='ip')
        parser_b = subparsers.add_parser('domain', help='Request passive DNS info on a domain')
        parser_b.add_argument('DOMAIN',  help='DOMAIN to be queried')
        parser_b.set_defaults(subcommand='domain')
        self.parser = parser


    def run(self, conf, args, plugins):
        dnsdb = Dnsdb(conf['Dnsdb']['key'])
        if 'subcommand' in args:
            if args.subcommand == "domain":
                results = dnsdb.search(name=args.DOMAIN)
                if results.status_code != 200:
                    print("Request failed : status code {}".format(results.status_code))
                else:
                    for r in results.records:
                        print("{}\t{}\t{}\t{}".format(
                            r['rrtype'],
                            r['time_first'],
                            r['time_last'],
                            "/ ".join(r['rdata'])
                        ))
            elif args.subcommand == "ip":
                results = dnsdb.search(ip=args.IP)
                if results.status_code != 200:
                    print("Request failed : status code {}".format(results.status_code))
                else:
                    for r in results.records:
                        print("{}\t{}\t{}\t{}".format(
                            r["rrtype"],
                            r["time_first"],
                            r["time_last"],
                            r["rdata"]
                        ))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



import requests
import urllib
from bs4 import BeautifulSoup
from harpoon.lib.utils import same_url

class Yandex(object):
    @staticmethod
    def search(req):
        '''
        Search for a request in Yandex and return results
        '''
        r = requests.get('https://yandex.ru/search/?text=%s' %
                urllib.parse.quote(req, safe='')
        )
        soup = BeautifulSoup(r.text, 'lxml')
        res = []
        for l in soup.find_all('li', class_='serp-item'):
            url = l.a['href']
            encoded_url = urllib.parse.quote(url, safe='')
            result = {
                'url': l.a['href'],
                'name': l.a.text,
            }
            text = l.find('div', class_="text-container")
            if(text):
                result['text'] = text.text
            popup = l.find_all('div', class_='popup2')
            if len(popup):
                for link in popup[0].find_all('a'):
                    if 'translate.yandex.ru' in link['href']:
                        if link['href'].startswith('http'):
                            result['translate'] = link['href']
                        else:
                            result['translate'] = 'http:' + link['href']
                    if 'hghltd.yandex.net' in link['href']:
                        result['cache'] = link['href']
            res.append(result)

        return res

    @staticmethod
    def download_cache(cache_url):
        '''
        Extract content from a cached Yandex url
        '''
        # FIXME: do not get date and url
        r = requests.get(cache_url)
        if r.status_code == 200:
            return {
                'success': True,
                'data': r.text[:-90],
                'cacheurl': cache_url,
            }
        else:
            return {'success': False}

    @staticmethod
    def cache(url):
        """
        Search for a cache url in yandex and if found get its content
        """
        # FIXME: miss obvious pages, like www.domain.com instead of domain.com
        res = Yandex.search(url)
        for i in res:
            if i['url'] == url:
                if 'cache' in i:
                    return Yandex.download_cache(i['cache'])
        return {'success': False }
